# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import copy
import os

# Import Salt Libs
import salt.modules.scsi as scsi
import salt.utils.path

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class ScsiTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.scsi
    """

    def setup_loader_modules(self):
        return {scsi: {}}

    def test_ls_(self):
        """
        Test for list SCSI devices, with details
        """
        lsscsi = {
            "stdout": "[0:0:0:0] disk HP LOGICAL VOLUME 6.68 /dev/sda [8:0]",
            "stderr": "",
            "retcode": 0,
        }

        lsscsi_size = {
            "stdout": "[0:0:0:0] disk HP LOGICAL VOLUME 6.68 /dev/sda [8:0] 1.20TB",
            "stderr": "",
            "retcode": 0,
        }

        result = {
            "[0:0:0:0]": {
                "major": "8",
                "lun": "0:0:0:0",
                "device": "/dev/sda",
                "model": "LOGICAL VOLUME 6.68",
                "minor": "0",
                "size": None,
            }
        }
        result_size = copy.deepcopy(result)
        result_size["[0:0:0:0]"]["size"] = "1.20TB"

        mock = MagicMock(return_value="/usr/bin/lsscsi")
        with patch.object(salt.utils.path, "which", mock):
            # get_size = True

            cmd_mock = MagicMock(return_value=lsscsi_size)
            with patch.dict(scsi.__salt__, {"cmd.run_all": cmd_mock}):
                self.assertDictEqual(scsi.ls_(), result_size)
                with patch.dict(
                    lsscsi_size, {"retcode": 1, "stderr": "An error occurred"}
                ):
                    self.assertEqual(scsi.ls_(), "An error occurred")
                with patch.dict(
                    lsscsi_size,
                    {"retcode": 1, "stderr": "lsscsi: invalid option -- 's'\nUsage:"},
                ):
                    self.assertEqual(
                        scsi.ls_(), "lsscsi: invalid option -- 's' - try get_size=False"
                    )

            # get_size = False
            cmd_mock = MagicMock(return_value=lsscsi)
            with patch.dict(scsi.__salt__, {"cmd.run_all": cmd_mock}):
                self.assertDictEqual(scsi.ls_(get_size=False), result)

        mock = MagicMock(return_value=None)
        with patch.object(salt.utils.path, "which", mock):
            self.assertEqual(
                scsi.ls_(), "scsi.ls not available - lsscsi command not found"
            )

    def test_rescan_all(self):
        """
        Test for list scsi devices
        """
        mock = MagicMock(side_effect=[False, True])
        with patch.object(os.path, "isdir", mock):
            self.assertEqual(scsi.rescan_all("host"), "Host host does not exist")

            with patch.dict(scsi.__salt__, {"cmd.run": MagicMock(return_value="A")}):
                self.assertListEqual(scsi.rescan_all("host"), ["A"])



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

from datetime import datetime

# Import Salt Libs
import salt.modules.redismod as redismod

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock
from tests.support.unit import TestCase


class Mockredis(object):
    """
    Mock redis class
    """

    class ConnectionError(Exception):
        """
        Mock ConnectionError class
        """


class MockConnect(object):
    """
    Mock Connect class
    """

    counter = 0

    def __init__(self):
        self.name = None
        self.pattern = None
        self.value = None
        self.key = None
        self.seconds = None
        self.timestamp = None
        self.field = None
        self.start = None
        self.stop = None
        self.master_host = None
        self.master_port = None

    @staticmethod
    def bgrewriteaof():
        """
        Mock bgrewriteaof method
        """
        return "A"

    @staticmethod
    def bgsave():
        """
        Mock bgsave method
        """
        return "A"

    def config_get(self, pattern):
        """
        Mock config_get method
        """
        self.pattern = pattern
        return "A"

    def config_set(self, name, value):
        """
        Mock config_set method
        """
        self.name = name
        self.value = value
        return "A"

    @staticmethod
    def dbsize():
        """
        Mock dbsize method
        """
        return "A"

    @staticmethod
    def delete():
        """
        Mock delete method
        """
        return "A"

    def exists(self, key):
        """
        Mock exists method
        """
        self.key = key
        return "A"

    def expire(self, key, seconds):
        """
        Mock expire method
        """
        self.key = key
        self.seconds = seconds
        return "A"

    def expireat(self, key, timestamp):
        """
        Mock expireat method
        """
        self.key = key
        self.timestamp = timestamp
        return "A"

    @staticmethod
    def flushall():
        """
        Mock flushall method
        """
        return "A"

    @staticmethod
    def flushdb():
        """
        Mock flushdb method
        """
        return "A"

    def get(self, key):
        """
        Mock get method
        """
        self.key = key
        return "A"

    def hget(self, key, field):
        """
        Mock hget method
        """
        self.key = key
        self.field = field
        return "A"

    def hgetall(self, key):
        """
        Mock hgetall method
        """
        self.key = key
        return "A"

    @staticmethod
    def info():
        """
        Mock info method
        """
        return "A"

    def keys(self, pattern):
        """
        Mock keys method
        """
        self.pattern = pattern
        return "A"

    def type(self, key):
        """
        Mock type method
        """
        self.key = key
        return "A"

    @staticmethod
    def lastsave():
        """
        Mock lastsave method
        """
        return datetime.now()

    def llen(self, key):
        """
        Mock llen method
        """
        self.key = key
        return "A"

    def lrange(self, key, start, stop):
        """
        Mock lrange method
        """
        self.key = key
        self.start = start
        self.stop = stop
        return "A"

    @staticmethod
    def ping():
        """
        Mock ping method
        """
        MockConnect.counter = MockConnect.counter + 1
        if MockConnect.counter == 1:
            return "A"
        elif MockConnect.counter in (2, 3, 5):
            raise Mockredis.ConnectionError("foo")

    @staticmethod
    def save():
        """
        Mock save method
        """
        return "A"

    def set(self, key, value):
        """
        Mock set method
        """
        self.key = key
        self.value = value
        return "A"

    @staticmethod
    def shutdown():
        """
        Mock shutdown method
        """
        return "A"

    def slaveof(self, master_host, master_port):
        """
        Mock slaveof method
        """
        self.master_host = master_host
        self.master_port = master_port
        return "A"

    def smembers(self, key):
        """
        Mock smembers method
        """
        self.key = key
        return "A"

    @staticmethod
    def time():
        """
        Mock time method
        """
        return "A"

    def zcard(self, key):
        """
        Mock zcard method
        """
        self.key = key
        return "A"

    def zrange(self, key, start, stop):
        """
        Mock zrange method
        """
        self.key = key
        self.start = start
        self.stop = stop
        return "A"


class RedismodTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.redismod
    """

    def setup_loader_modules(self):
        return {
            redismod: {
                "redis": Mockredis,
                "_connect": MagicMock(return_value=MockConnect()),
            }
        }

    def test_bgrewriteaof(self):
        """
        Test to asynchronously rewrite the append-only file
        """
        self.assertEqual(redismod.bgrewriteaof(), "A")

    def test_bgsave(self):
        """
        Test to asynchronously save the dataset to disk
        """
        self.assertEqual(redismod.bgsave(), "A")

    def test_config_get(self):
        """
        Test to get redis server configuration values
        """
        self.assertEqual(redismod.config_get("*"), "A")

    def test_config_set(self):
        """
        Test to set redis server configuration values
        """
        self.assertEqual(redismod.config_set("name", "value"), "A")

    def test_dbsize(self):
        """
        Test to return the number of keys in the selected database
        """
        self.assertEqual(redismod.dbsize(), "A")

    def test_delete(self):
        """
        Test to deletes the keys from redis, returns number of keys deleted
        """
        self.assertEqual(redismod.delete(), "A")

    def test_exists(self):
        """
        Test to return true if the key exists in redis
        """
        self.assertEqual(redismod.exists("key"), "A")

    def test_expire(self):
        """
        Test to set a keys time to live in seconds
        """
        self.assertEqual(redismod.expire("key", "seconds"), "A")

    def test_expireat(self):
        """
        Test to set a keys expire at given UNIX time
        """
        self.assertEqual(redismod.expireat("key", "timestamp"), "A")

    def test_flushall(self):
        """
        Test to remove all keys from all databases
        """
        self.assertEqual(redismod.flushall(), "A")

    def test_flushdb(self):
        """
        Test to remove all keys from the selected database
        """
        self.assertEqual(redismod.flushdb(), "A")

    def test_get_key(self):
        """
        Test to get redis key value
        """
        self.assertEqual(redismod.get_key("key"), "A")

    def test_hget(self):
        """
        Test to get specific field value from a redis hash, returns dict
        """
        self.assertEqual(redismod.hget("key", "field"), "A")

    def test_hgetall(self):
        """
        Test to get all fields and values from a redis hash, returns dict
        """
        self.assertEqual(redismod.hgetall("key"), "A")

    def test_info(self):
        """
        Test to get information and statistics about the server
        """
        self.assertEqual(redismod.info(), "A")

    def test_keys(self):
        """
        Test to get redis keys, supports glob style patterns
        """
        self.assertEqual(redismod.keys("pattern"), "A")

    def test_key_type(self):
        """
        Test to get redis key type
        """
        self.assertEqual(redismod.key_type("key"), "A")

    def test_lastsave(self):
        """
        Test to get the UNIX time in seconds of the last successful
        save to disk
        """
        self.assertTrue(redismod.lastsave())

    def test_llen(self):
        """
        Test to get the length of a list in Redis
        """
        self.assertEqual(redismod.llen("key"), "A")

    def test_lrange(self):
        """
        Test to get a range of values from a list in Redis
        """
        self.assertEqual(redismod.lrange("key", "start", "stop"), "A")

    def test_ping(self):
        """
        Test to ping the server, returns False on connection errors
        """
        self.assertEqual(redismod.ping(), "A")

        self.assertFalse(redismod.ping())

    def test_save(self):
        """
        Test to synchronously save the dataset to disk
        """
        self.assertEqual(redismod.save(), "A")

    def test_set_key(self):
        """
        Test to set redis key value
        """
        self.assertEqual(redismod.set_key("key", "value"), "A")

    def test_shutdown(self):
        """
        Test to synchronously save the dataset to disk and then
        shut down the server
        """
        self.assertFalse(redismod.shutdown())

        self.assertTrue(redismod.shutdown())

        self.assertFalse(redismod.shutdown())

    def test_slaveof(self):
        """
        Test to make the server a slave of another instance, or
        promote it as master
        """
        self.assertEqual(redismod.slaveof("master_host", "master_port"), "A")

    def test_smembers(self):
        """
        Test to get members in a Redis set
        """
        self.assertListEqual(redismod.smembers("key"), ["A"])

    def test_time(self):
        """
        Test to return the current server UNIX time in seconds
        """
        self.assertEqual(redismod.time(), "A")

    def test_zcard(self):
        """
        Test to get the length of a sorted set in Redis
        """
        self.assertEqual(redismod.zcard("key"), "A")

    def test_zrange(self):
        """
        Test to get a range of values from a sorted set in Redis by index
        """
        self.assertEqual(redismod.zrange("key", "start", "stop"), "A")



# -*- coding: utf-8 -*-
"""
    :codeauthor: Alexander Schwartz <alexander.schwartz@gmx.net>
"""

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import os

# Import Salt Libs
import salt.states.archive as archive
import salt.utils.platform
from salt.ext.six.moves import zip  # pylint: disable=import-error,redefined-builtin

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


def _isfile_side_effect(path):
    """
    MagicMock side_effect for os.path.isfile(). We don't want to use dict.get
    here because we want the test to fail if there's a path we haven't
    accounted for, so that we can add it.

    NOTE: This may fall over on some platforms if /usr/bin/tar does not exist.
    If so, just add an entry in the dictionary for the path being used for tar.
    """
    return {
        "/tmp/foo.tar.gz": True,
        "c:\\tmp\\foo.tar.gz": True,
        "/private/tmp/foo.tar.gz": True,
        "/tmp/out": False,
        "\\tmp\\out": False,
        "/usr/bin/tar": True,
        "/bin/tar": True,
        "/tmp/test_extracted_tar": False,
        "c:\\tmp\\test_extracted_tar": False,
        "/private/tmp/test_extracted_tar": False,
    }[path.lower() if salt.utils.platform.is_windows() else path]


class ArchiveTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {
            archive: {
                "__grains__": {"os": "FooOS!"},
                "__opts__": {"cachedir": "/tmp", "test": False, "hash_type": "sha256"},
                "__env__": "test",
            }
        }

    def test_extracted_tar(self):
        """
        archive.extracted tar options
        """

        if salt.utils.platform.is_windows():
            source = "c:\\tmp\\foo.tar.gz"
            tmp_dir = "c:\\tmp\\test_extracted_tar"
        elif salt.utils.platform.is_darwin():
            source = "/private/tmp/foo.tar.gz"
            tmp_dir = "/private/tmp/test_extracted_tar"
        else:
            source = "/tmp/foo.tar.gz"
            tmp_dir = "/tmp/test_extracted_tar"
        test_tar_opts = [
            "--no-anchored foo",
            "v -p --opt",
            "-v -p",
            "--long-opt -z",
            "z -v -weird-long-opt arg",
        ]
        ret_tar_opts = [
            ["tar", "xv", "--no-anchored", "foo", "-f"],
            ["tar", "xv", "-p", "--opt", "-f"],
            ["tar", "xv", "-p", "-f"],
            ["tar", "xv", "--long-opt", "-z", "-f"],
            ["tar", "xvz", "-weird-long-opt", "arg", "-f"],
        ]

        mock_true = MagicMock(return_value=True)
        mock_false = MagicMock(return_value=False)
        ret = {
            "stdout": ["cheese", "ham", "saltines"],
            "stderr": "biscuits",
            "retcode": "31337",
            "pid": "1337",
        }
        mock_run = MagicMock(return_value=ret)
        mock_source_list = MagicMock(return_value=(source, None))
        state_single_mock = MagicMock(return_value={"local": {"result": True}})
        list_mock = MagicMock(
            return_value={
                "dirs": [],
                "files": ["cheese", "saltines"],
                "links": ["ham"],
                "top_level_dirs": [],
                "top_level_files": ["cheese", "saltines"],
                "top_level_links": ["ham"],
            }
        )
        isfile_mock = MagicMock(side_effect=_isfile_side_effect)

        with patch.dict(
            archive.__opts__,
            {"test": False, "cachedir": tmp_dir, "hash_type": "sha256"},
        ), patch.dict(
            archive.__salt__,
            {
                "file.directory_exists": mock_false,
                "file.file_exists": mock_false,
                "state.single": state_single_mock,
                "file.makedirs": mock_true,
                "cmd.run_all": mock_run,
                "archive.list": list_mock,
                "file.source_list": mock_source_list,
            },
        ), patch.dict(
            archive.__states__, {"file.directory": mock_true}
        ), patch.object(
            os.path, "isfile", isfile_mock
        ), patch(
            "salt.utils.path.which", MagicMock(return_value=True)
        ):

            for test_opts, ret_opts in zip(test_tar_opts, ret_tar_opts):
                archive.extracted(
                    tmp_dir, source, options=test_opts, enforce_toplevel=False
                )
                ret_opts.append(source)
                mock_run.assert_called_with(
                    ret_opts, cwd=tmp_dir + os.sep, python_shell=False
                )

    def test_tar_gnutar(self):
        """
        Tests the call of extraction with gnutar
        """
        gnutar = MagicMock(return_value="tar (GNU tar)")
        source = "/tmp/foo.tar.gz"
        mock_false = MagicMock(return_value=False)
        mock_true = MagicMock(return_value=True)
        state_single_mock = MagicMock(return_value={"local": {"result": True}})
        run_all = MagicMock(
            return_value={"retcode": 0, "stdout": "stdout", "stderr": "stderr"}
        )
        mock_source_list = MagicMock(return_value=(source, None))
        list_mock = MagicMock(
            return_value={
                "dirs": [],
                "files": ["stdout"],
                "links": [],
                "top_level_dirs": [],
                "top_level_files": ["stdout"],
                "top_level_links": [],
            }
        )
        isfile_mock = MagicMock(side_effect=_isfile_side_effect)

        with patch.dict(
            archive.__salt__,
            {
                "cmd.run": gnutar,
                "file.directory_exists": mock_false,
                "file.file_exists": mock_false,
                "state.single": state_single_mock,
                "file.makedirs": mock_true,
                "cmd.run_all": run_all,
                "archive.list": list_mock,
                "file.source_list": mock_source_list,
            },
        ), patch.dict(archive.__states__, {"file.directory": mock_true}), patch.object(
            os.path, "isfile", isfile_mock
        ), patch(
            "salt.utils.path.which", MagicMock(return_value=True)
        ):
            ret = archive.extracted(
                os.path.join(os.sep + "tmp", "out"),
                source,
                options="xvzf",
                enforce_toplevel=False,
                keep=True,
            )
            self.assertEqual(ret["changes"]["extracted_files"], ["stdout"])

    def test_tar_bsdtar(self):
        """
        Tests the call of extraction with bsdtar
        """
        bsdtar = MagicMock(return_value="tar (bsdtar)")
        source = "/tmp/foo.tar.gz"
        mock_false = MagicMock(return_value=False)
        mock_true = MagicMock(return_value=True)
        state_single_mock = MagicMock(return_value={"local": {"result": True}})
        run_all = MagicMock(
            return_value={"retcode": 0, "stdout": "stdout", "stderr": "stderr"}
        )
        mock_source_list = MagicMock(return_value=(source, None))
        list_mock = MagicMock(
            return_value={
                "dirs": [],
                "files": ["stderr"],
                "links": [],
                "top_level_dirs": [],
                "top_level_files": ["stderr"],
                "top_level_links": [],
            }
        )
        isfile_mock = MagicMock(side_effect=_isfile_side_effect)

        with patch.dict(
            archive.__salt__,
            {
                "cmd.run": bsdtar,
                "file.directory_exists": mock_false,
                "file.file_exists": mock_false,
                "state.single": state_single_mock,
                "file.makedirs": mock_true,
                "cmd.run_all": run_all,
                "archive.list": list_mock,
                "file.source_list": mock_source_list,
            },
        ), patch.dict(archive.__states__, {"file.directory": mock_true}), patch.object(
            os.path, "isfile", isfile_mock
        ), patch(
            "salt.utils.path.which", MagicMock(return_value=True)
        ):
            ret = archive.extracted(
                os.path.join(os.sep + "tmp", "out"),
                source,
                options="xvzf",
                enforce_toplevel=False,
                keep=True,
            )
            self.assertEqual(ret["changes"]["extracted_files"], ["stderr"])

    def test_tar_bsdtar_with_trim_output(self):
        """
        Tests the call of extraction with bsdtar with trim_output
        """
        bsdtar = MagicMock(return_value="tar (bsdtar)")
        source = "/tmp/foo.tar.gz"
        mock_false = MagicMock(return_value=False)
        mock_true = MagicMock(return_value=True)
        state_single_mock = MagicMock(return_value={"local": {"result": True}})
        run_all = MagicMock(
            return_value={"retcode": 0, "stdout": "stdout", "stderr": "stderr"}
        )
        mock_source_list = MagicMock(return_value=(source, None))
        list_mock = MagicMock(
            return_value={
                "dirs": [],
                "files": ["stderr"],
                "links": [],
                "top_level_dirs": [],
                "top_level_files": ["stderr"],
                "top_level_links": [],
            }
        )
        isfile_mock = MagicMock(side_effect=_isfile_side_effect)

        with patch.dict(
            archive.__salt__,
            {
                "cmd.run": bsdtar,
                "file.directory_exists": mock_false,
                "file.file_exists": mock_false,
                "state.single": state_single_mock,
                "file.makedirs": mock_true,
                "cmd.run_all": run_all,
                "archive.list": list_mock,
                "file.source_list": mock_source_list,
            },
        ), patch.dict(archive.__states__, {"file.directory": mock_true}), patch.object(
            os.path, "isfile", isfile_mock
        ), patch(
            "salt.utils.path.which", MagicMock(return_value=True)
        ):
            ret = archive.extracted(
                os.path.join(os.sep + "tmp", "out"),
                source,
                options="xvzf",
                enforce_toplevel=False,
                keep_source=True,
                trim_output=1,
            )
            self.assertEqual(ret["changes"]["extracted_files"], ["stderr"])
            self.assertTrue(
                ret["comment"].endswith("Output was trimmed to 1 number of lines")
            )

    def test_extracted_when_if_missing_path_exists(self):
        """
        When if_missing exists, we should exit without making any changes.

        NOTE: We're not mocking the __salt__ dunder because if we actually run
        any functions from that dunder, we're doing something wrong. So, in
        those cases we'll just let it raise a KeyError and cause the test to
        fail.
        """
        name = if_missing = "/tmp/foo"
        source = "salt://foo.bar.tar"
        with patch.object(os.path, "exists", MagicMock(return_value=True)):
            ret = archive.extracted(name, source=source, if_missing=if_missing)
            self.assertTrue(ret["result"], ret)
            self.assertEqual(ret["comment"], "Path {0} exists".format(if_missing))

    def test_clean_parent_conflict(self):
        """
        Tests the call of extraction with gnutar with both clean_parent plus clean set to True
        """
        gnutar = MagicMock(return_value="tar (GNU tar)")
        source = "/tmp/foo.tar.gz"
        ret_comment = "Only one of 'clean' and 'clean_parent' can be set to True"
        mock_false = MagicMock(return_value=False)
        mock_true = MagicMock(return_value=True)
        state_single_mock = MagicMock(return_value={"local": {"result": True}})
        run_all = MagicMock(
            return_value={"retcode": 0, "stdout": "stdout", "stderr": "stderr"}
        )
        mock_source_list = MagicMock(return_value=(source, None))
        list_mock = MagicMock(
            return_value={
                "dirs": [],
                "files": ["stdout"],
                "links": [],
                "top_level_dirs": [],
                "top_level_files": ["stdout"],
                "top_level_links": [],
            }
        )
        isfile_mock = MagicMock(side_effect=_isfile_side_effect)

        with patch.dict(
            archive.__salt__,
            {
                "cmd.run": gnutar,
                "file.directory_exists": mock_false,
                "file.file_exists": mock_false,
                "state.single": state_single_mock,
                "file.makedirs": mock_true,
                "cmd.run_all": run_all,
                "archive.list": list_mock,
                "file.source_list": mock_source_list,
            },
        ), patch.dict(archive.__states__, {"file.directory": mock_true}), patch.object(
            os.path, "isfile", isfile_mock
        ), patch(
            "salt.utils.path.which", MagicMock(return_value=True)
        ):
            ret = archive.extracted(
                os.path.join(os.sep + "tmp", "out"),
                source,
                options="xvzf",
                enforce_toplevel=False,
                clean=True,
                clean_parent=True,
                keep=True,
            )
            self.assertEqual(ret["result"], False)
            self.assertEqual(ret["changes"], {})
            self.assertEqual(ret["comment"], ret_comment)

    def test_skip_files_list_verify_conflict(self):
        """
        Tests the call of extraction  with both skip_files_list_verify and skip_verify set to True
        """
        gnutar = MagicMock(return_value="tar (GNU tar)")
        source = "/tmp/foo.tar.gz"
        ret_comment = (
            'Only one of "skip_files_list_verify" and "skip_verify" can be set to True'
        )
        mock_false = MagicMock(return_value=False)
        mock_true = MagicMock(return_value=True)
        state_single_mock = MagicMock(return_value={"local": {"result": True}})
        run_all = MagicMock(
            return_value={"retcode": 0, "stdout": "stdout", "stderr": "stderr"}
        )
        mock_source_list = MagicMock(return_value=(source, None))
        list_mock = MagicMock(
            return_value={
                "dirs": [],
                "files": ["stdout"],
                "links": [],
                "top_level_dirs": [],
                "top_level_files": ["stdout"],
                "top_level_links": [],
            }
        )
        isfile_mock = MagicMock(side_effect=_isfile_side_effect)

        with patch.dict(
            archive.__salt__,
            {
                "cmd.run": gnutar,
                "file.directory_exists": mock_false,
                "file.file_exists": mock_false,
                "state.single": state_single_mock,
                "file.makedirs": mock_true,
                "cmd.run_all": run_all,
                "archive.list": list_mock,
                "file.source_list": mock_source_list,
            },
        ), patch.dict(archive.__states__, {"file.directory": mock_true}), patch.object(
            os.path, "isfile", isfile_mock
        ), patch(
            "salt.utils.path.which", MagicMock(return_value=True)
        ):
            ret = archive.extracted(
                os.path.join(os.sep + "tmp", "out"),
                source,
                options="xvzf",
                enforce_toplevel=False,
                clean=True,
                skip_files_list_verify=True,
                skip_verify=True,
                keep=True,
            )
            self.assertEqual(ret["result"], False)
            self.assertEqual(ret["changes"], {})
            self.assertEqual(ret["comment"], ret_comment)

    def test_skip_files_list_verify_success(self):
        """
        Test that if the local and expected source hash are the same we won't do anything.
        """

        if salt.utils.platform.is_windows():
            source = "c:\\tmp\\foo.tar.gz"
            tmp_dir = "c:\\tmp\\test_extracted_tar"
        elif salt.utils.platform.is_darwin():
            source = "/private/tmp/foo.tar.gz"
            tmp_dir = "/private/tmp/test_extracted_tar"
        else:
            source = "/tmp/foo.tar.gz"
            tmp_dir = "/tmp/test_extracted_tar"

        expected_comment = (
            "Archive {} existing source sum is the same as the "
            "expected one and skip_files_list_verify argument "
            "was set to True. Extraction is not needed".format(source)
        )
        expected_ret = {
            "name": tmp_dir,
            "result": True,
            "changes": {},
            "comment": expected_comment,
        }
        mock_true = MagicMock(return_value=True)
        mock_false = MagicMock(return_value=False)
        mock_cached = MagicMock(return_value="{}/{}".format(tmp_dir, source))
        source_sum = {"hsum": "testhash", "hash_type": "sha256"}
        mock_hash = MagicMock(return_value=source_sum)
        mock_source_list = MagicMock(return_value=(source, None))
        isfile_mock = MagicMock(side_effect=_isfile_side_effect)

        with patch("salt.states.archive._read_cached_checksum", mock_hash):
            with patch.dict(
                archive.__opts__,
                {"test": False, "cachedir": tmp_dir, "hash_type": "sha256"},
            ), patch.dict(
                archive.__salt__,
                {
                    "file.directory_exists": mock_false,
                    "file.get_source_sum": mock_hash,
                    "file.check_hash": mock_true,
                    "cp.is_cached": mock_cached,
                    "file.source_list": mock_source_list,
                },
            ), patch.object(
                os.path, "isfile", isfile_mock
            ):

                ret = archive.extracted(
                    tmp_dir,
                    source,
                    source_hash="testhash",
                    skip_files_list_verify=True,
                    enforce_toplevel=False,
                )
                self.assertDictEqual(ret, expected_ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import inspect

# Import Salt Libs
import salt.modules.defaults as defaults

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class DefaultsTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.defaults
    """

    def setup_loader_modules(self):
        return {defaults: {}}

    def test_get_mock(self):
        """
        Test if it execute a defaults client run and return a dict
        """
        with patch.object(inspect, "stack", MagicMock(return_value=[])), patch(
            "salt.modules.defaults.get",
            MagicMock(return_value={"users": {"root": [0]}}),
        ):
            self.assertEqual(defaults.get("core:users:root"), {"users": {"root": [0]}})

    def test_merge_with_list_merging(self):
        """
        Test deep merging of dicts with merge_lists enabled.
        """

        src_dict = {
            "string_key": "string_val_src",
            "list_key": ["list_val_src"],
            "dict_key": {"dict_key_src": "dict_val_src"},
        }

        dest_dict = {
            "string_key": "string_val_dest",
            "list_key": ["list_val_dest"],
            "dict_key": {"dict_key_dest": "dict_val_dest"},
        }

        merged_dict = {
            "string_key": "string_val_src",
            "list_key": ["list_val_dest", "list_val_src"],
            "dict_key": {
                "dict_key_dest": "dict_val_dest",
                "dict_key_src": "dict_val_src",
            },
        }

        defaults.merge(dest_dict, src_dict, merge_lists=True)
        self.assertEqual(dest_dict, merged_dict)

    def test_merge_without_list_merging(self):
        """
        Test deep merging of dicts with merge_lists disabled.
        """

        src = {
            "string_key": "string_val_src",
            "list_key": ["list_val_src"],
            "dict_key": {"dict_key_src": "dict_val_src"},
        }

        dest = {
            "string_key": "string_val_dest",
            "list_key": ["list_val_dest"],
            "dict_key": {"dict_key_dest": "dict_val_dest"},
        }

        merged = {
            "string_key": "string_val_src",
            "list_key": ["list_val_src"],
            "dict_key": {
                "dict_key_dest": "dict_val_dest",
                "dict_key_src": "dict_val_src",
            },
        }

        defaults.merge(dest, src, merge_lists=False)
        self.assertEqual(dest, merged)

    def test_merge_not_in_place(self):
        """
        Test deep merging of dicts not in place.
        """

        src = {"nested_dict": {"A": "A"}}

        dest = {"nested_dict": {"B": "B"}}

        dest_orig = {"nested_dict": {"B": "B"}}

        merged = {"nested_dict": {"A": "A", "B": "B"}}

        final = defaults.merge(dest, src, in_place=False)
        self.assertEqual(dest, dest_orig)
        self.assertEqual(final, merged)

    def test_deepcopy(self):
        """
        Test a deep copy of object.
        """

        src = {"A": "A", "B": "B"}

        dist = defaults.deepcopy(src)
        dist.update({"C": "C"})

        result = {"A": "A", "B": "B", "C": "C"}

        self.assertFalse(src == dist)
        self.assertTrue(dist == result)

    def test_update_in_place(self):
        """
        Test update with defaults values in place.
        """

        group01 = {
            "defaults": {"enabled": True, "extra": ["test", "stage"]},
            "nodes": {"host01": {"index": "foo", "upstream": "bar"}},
        }

        host01 = {
            "enabled": True,
            "index": "foo",
            "upstream": "bar",
            "extra": ["test", "stage"],
        }

        defaults.update(group01["nodes"], group01["defaults"])
        self.assertEqual(group01["nodes"]["host01"], host01)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import os

# Import Salt Libs
import salt.states.mount as mount

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class MountTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.mount
    """

    def setup_loader_modules(self):
        return {mount: {}}

    # 'mounted' function tests: 1

    def test_mounted(self):
        """
        Test to verify that a device is mounted.
        """
        name = os.path.realpath("/mnt/sdb")
        device = os.path.realpath("/dev/sdb5")
        fstype = "xfs"

        name2 = os.path.realpath("/mnt/cifs")
        device2 = "//SERVER/SHARE/"
        fstype2 = "cifs"
        opts2 = ["noowners"]
        superopts2 = ["uid=510", "gid=100", "username=cifsuser", "domain=cifsdomain"]

        name3 = os.path.realpath("/mnt/jfs2")
        device3 = "/dev/hd1"
        fstype3 = "jfs2"
        opts3 = [""]
        superopts3 = ["uid=510", "gid=100", "username=jfs2user", "domain=jfs2sdomain"]

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        mock = MagicMock(
            side_effect=[
                "new",
                "present",
                "new",
                "change",
                "bad config",
                "salt",
                "present",
            ]
        )
        mock_t = MagicMock(return_value=True)
        mock_f = MagicMock(return_value=False)
        mock_ret = MagicMock(return_value={"retcode": 1})
        mock_mnt = MagicMock(
            return_value={
                name: {"device": device, "opts": [], "superopts": []},
                name2: {"device": device2, "opts": opts2, "superopts": superopts2},
                name3: {"device": device3, "opts": opts3, "superopts": superopts3},
            }
        )
        mock_aixfs_retn = MagicMock(return_value="present")

        mock_emt = MagicMock(return_value={})
        mock_str = MagicMock(return_value="salt")
        mock_user = MagicMock(return_value={"uid": 510})
        mock_group = MagicMock(return_value={"gid": 100})
        mock_read_cache = MagicMock(return_value={})
        mock_write_cache = MagicMock(return_value=True)
        with patch.dict(mount.__grains__, {"os": "Darwin"}):
            with patch.dict(
                mount.__salt__,
                {
                    "mount.active": mock_mnt,
                    "cmd.run_all": mock_ret,
                    "mount.umount": mock_f,
                },
            ), patch("os.path.exists", MagicMock(return_value=True)):
                comt = "Unable to find device with label /dev/sdb5."
                ret.update({"comment": comt})
                self.assertDictEqual(
                    mount.mounted(name, "LABEL=/dev/sdb5", fstype), ret
                )

                with patch.dict(mount.__opts__, {"test": True}):
                    comt = (
                        "Remount would be forced because" " options (noowners) changed"
                    )
                    ret.update({"comment": comt, "result": None})
                    self.assertDictEqual(mount.mounted(name, device, fstype), ret)

                with patch.dict(mount.__opts__, {"test": False}):
                    comt = "Unable to unmount {0}: False.".format(name)
                    umount = (
                        "Forced unmount and mount because" " options (noowners) changed"
                    )
                    ret.update(
                        {
                            "comment": comt,
                            "result": False,
                            "changes": {"umount": umount},
                        }
                    )
                    self.assertDictEqual(mount.mounted(name, device, "nfs"), ret)

                    umount1 = (
                        "Forced unmount because devices don't match. "
                        "Wanted: {0}, current: {1}, {1}".format(
                            os.path.realpath("/dev/sdb6"), device
                        )
                    )
                    comt = "Unable to unmount"
                    ret.update(
                        {
                            "comment": comt,
                            "result": None,
                            "changes": {"umount": umount1},
                        }
                    )
                    self.assertDictEqual(
                        mount.mounted(
                            name, os.path.realpath("/dev/sdb6"), fstype, opts=[]
                        ),
                        ret,
                    )

                with patch.dict(
                    mount.__salt__,
                    {
                        "mount.active": mock_emt,
                        "mount.mount": mock_str,
                        "mount.set_automaster": mock,
                    },
                ):
                    with patch.dict(mount.__opts__, {"test": True}), patch(
                        "os.path.exists", MagicMock(return_value=False)
                    ):
                        comt = "{0} does not exist and would not be created".format(
                            name
                        )
                        ret.update({"comment": comt, "changes": {}})
                        self.assertDictEqual(mount.mounted(name, device, fstype), ret)

                    with patch.dict(mount.__opts__, {"test": False}):
                        with patch.object(os.path, "exists", mock_f):
                            comt = "Mount directory is not present"
                            ret.update({"comment": comt, "result": False})
                            self.assertDictEqual(
                                mount.mounted(name, device, fstype), ret
                            )

                        with patch.object(os.path, "exists", mock_t):
                            comt = "Mount directory is not present"
                            ret.update({"comment": "salt", "result": False})
                            self.assertDictEqual(
                                mount.mounted(name, device, fstype), ret
                            )

                    with patch.dict(mount.__opts__, {"test": True}), patch(
                        "os.path.exists", MagicMock(return_value=False)
                    ):
                        comt = (
                            "{0} does not exist and would neither be created nor mounted. "
                            "{0} needs to be written to the fstab in order to be made persistent.".format(
                                name
                            )
                        )
                        ret.update({"comment": comt, "result": None})
                        self.assertDictEqual(
                            mount.mounted(name, device, fstype, mount=False), ret
                        )

                    with patch.dict(mount.__opts__, {"test": False}), patch(
                        "os.path.exists", MagicMock(return_value=False)
                    ):
                        comt = (
                            "{0} not present and not mounted. "
                            "Entry already exists in the fstab.".format(name)
                        )
                        ret.update({"comment": comt, "result": True})
                        self.assertDictEqual(
                            mount.mounted(name, device, fstype, mount=False), ret
                        )

                        comt = (
                            "{0} not present and not mounted. "
                            "Added new entry to the fstab.".format(name)
                        )
                        ret.update(
                            {
                                "comment": comt,
                                "result": True,
                                "changes": {"persist": "new"},
                            }
                        )
                        self.assertDictEqual(
                            mount.mounted(name, device, fstype, mount=False), ret
                        )

                        comt = (
                            "{0} not present and not mounted. "
                            "Updated the entry in the fstab.".format(name)
                        )
                        ret.update(
                            {
                                "comment": comt,
                                "result": True,
                                "changes": {"persist": "update"},
                            }
                        )
                        self.assertDictEqual(
                            mount.mounted(name, device, fstype, mount=False), ret
                        )

                        comt = (
                            "{0} not present and not mounted. "
                            "However, the fstab was not found.".format(name)
                        )
                        ret.update({"comment": comt, "result": False, "changes": {}})
                        self.assertDictEqual(
                            mount.mounted(name, device, fstype, mount=False), ret
                        )

                        comt = "{0} not present and not mounted".format(name)
                        ret.update({"comment": comt, "result": True, "changes": {}})
                        self.assertDictEqual(
                            mount.mounted(name, device, fstype, mount=False), ret
                        )

        # Test no change for uid provided as a name #25293
        with patch.dict(mount.__grains__, {"os": "CentOS"}):
            with patch.dict(
                mount.__salt__,
                {
                    "mount.active": mock_mnt,
                    "mount.mount": mock_str,
                    "mount.umount": mock_f,
                    "mount.read_mount_cache": mock_read_cache,
                    "mount.write_mount_cache": mock_write_cache,
                    "mount.set_fstab": mock,
                    "user.info": mock_user,
                    "group.info": mock_group,
                },
            ):
                with patch.dict(mount.__opts__, {"test": True}):
                    with patch.object(os.path, "exists", mock_t):
                        comt = "Target was already mounted. Entry already exists in the fstab."
                        ret.update({"name": name2, "result": True})
                        ret.update({"comment": comt, "changes": {}})
                        self.assertDictEqual(
                            mount.mounted(
                                name2,
                                device2,
                                fstype2,
                                opts=["uid=user1", "gid=group1"],
                            ),
                            ret,
                        )

        with patch.dict(mount.__grains__, {"os": "AIX"}):
            with patch.dict(
                mount.__salt__,
                {
                    "mount.active": mock_mnt,
                    "mount.mount": mock_str,
                    "mount.umount": mock_f,
                    "mount.read_mount_cache": mock_read_cache,
                    "mount.write_mount_cache": mock_write_cache,
                    "mount.set_filesystems": mock_aixfs_retn,
                    "user.info": mock_user,
                    "group.info": mock_group,
                },
            ):
                with patch.dict(mount.__opts__, {"test": True}):
                    with patch.object(os.path, "exists", mock_t):
                        comt = "Target was already mounted. Entry already exists in the fstab."
                        ret.update({"name": name3, "result": True})
                        ret.update({"comment": comt, "changes": {}})
                        self.assertDictEqual(
                            mount.mounted(
                                name3,
                                device3,
                                fstype3,
                                opts=["uid=user1", "gid=group1"],
                            ),
                            ret,
                        )

    # 'swap' function tests: 1

    def test_swap(self):
        """
        Test to activates a swap device.
        """
        name = "/mnt/sdb"

        ret = {"name": name, "result": None, "comment": "", "changes": {}}

        mock = MagicMock(side_effect=["present", "new", "change", "bad config"])
        mock_f = MagicMock(return_value=False)
        mock_swp = MagicMock(return_value=[name])
        mock_fs = MagicMock(return_value={"none": {"device": name, "fstype": "xfs"}})
        mock_fs_diff = MagicMock(
            return_value={"none": {"device": "something_else", "fstype": "xfs"}}
        )
        mock_aixfs = MagicMock(return_value={name: {"dev": name, "fstype": "jfs2"}})
        mock_emt = MagicMock(return_value={})
        with patch.dict(mount.__grains__, {"os": "test"}):
            with patch.dict(
                mount.__salt__,
                {
                    "mount.swaps": mock_swp,
                    "mount.fstab": mock_fs_diff,
                    "file.is_link": mock_f,
                },
            ):
                with patch.dict(mount.__opts__, {"test": True}):
                    comt = (
                        "Swap {0} is set to be added to the "
                        "fstab and to be activated".format(name)
                    )
                    ret.update({"comment": comt})
                    self.assertDictEqual(mount.swap(name), ret)

                with patch.dict(mount.__opts__, {"test": False}):
                    comt = "Swap {0} already active".format(name)
                    ret.update({"comment": comt, "result": True})
                    self.assertDictEqual(mount.swap(name, persist=False), ret)

                    with patch.dict(
                        mount.__salt__,
                        {"mount.fstab": mock_emt, "mount.set_fstab": mock},
                    ):
                        comt = "Swap {0} already active".format(name)
                        ret.update({"comment": comt, "result": True})
                        self.assertDictEqual(mount.swap(name), ret)

                        comt = (
                            "Swap /mnt/sdb already active. "
                            "Added new entry to the fstab."
                        )
                        ret.update(
                            {
                                "comment": comt,
                                "result": True,
                                "changes": {"persist": "new"},
                            }
                        )
                        self.assertDictEqual(mount.swap(name), ret)

                        comt = (
                            "Swap /mnt/sdb already active. "
                            "Updated the entry in the fstab."
                        )
                        ret.update(
                            {
                                "comment": comt,
                                "result": True,
                                "changes": {"persist": "update"},
                            }
                        )
                        self.assertDictEqual(mount.swap(name), ret)

                        comt = (
                            "Swap /mnt/sdb already active. "
                            "However, the fstab was not found."
                        )
                        ret.update({"comment": comt, "result": False, "changes": {}})
                        self.assertDictEqual(mount.swap(name), ret)

        ret = {"name": name, "result": None, "comment": "", "changes": {}}

        mock = MagicMock(side_effect=["present", "new", "change", "bad config"])
        mock_emt = MagicMock(return_value={})
        with patch.dict(mount.__grains__, {"os": "test"}):
            with patch.dict(
                mount.__salt__,
                {
                    "mount.swaps": mock_swp,
                    "mount.fstab": mock_fs,
                    "file.is_link": mock_f,
                },
            ):
                with patch.dict(mount.__opts__, {"test": True}):
                    comt = "Swap {0} already active".format(name)
                    ret.update({"comment": comt, "result": True})
                    self.assertDictEqual(mount.swap(name), ret)

                with patch.dict(mount.__opts__, {"test": False}):
                    comt = "Swap {0} already active".format(name)
                    ret.update({"comment": comt, "result": True})
                    self.assertDictEqual(mount.swap(name), ret)

                    with patch.dict(
                        mount.__salt__,
                        {"mount.fstab": mock_emt, "mount.set_fstab": mock},
                    ):
                        comt = "Swap {0} already active".format(name)
                        ret.update({"comment": comt, "result": True})
                        self.assertDictEqual(mount.swap(name), ret)

                        comt = (
                            "Swap /mnt/sdb already active. "
                            "Added new entry to the fstab."
                        )
                        ret.update(
                            {
                                "comment": comt,
                                "result": True,
                                "changes": {"persist": "new"},
                            }
                        )
                        self.assertDictEqual(mount.swap(name), ret)

                        comt = (
                            "Swap /mnt/sdb already active. "
                            "Updated the entry in the fstab."
                        )
                        ret.update(
                            {
                                "comment": comt,
                                "result": True,
                                "changes": {"persist": "update"},
                            }
                        )
                        self.assertDictEqual(mount.swap(name), ret)

                        comt = (
                            "Swap /mnt/sdb already active. "
                            "However, the fstab was not found."
                        )
                        ret.update({"comment": comt, "result": False, "changes": {}})
                        self.assertDictEqual(mount.swap(name), ret)

        with patch.dict(mount.__grains__, {"os": "AIX"}):
            with patch.dict(
                mount.__salt__,
                {
                    "mount.swaps": mock_swp,
                    "mount.filesystems": mock_aixfs,
                    "file.is_link": mock_f,
                },
            ):
                with patch.dict(mount.__opts__, {"test": True}):
                    comt = "Swap {0} already active".format(name)
                    ret.update({"comment": comt, "result": True})
                    self.assertDictEqual(mount.swap(name), ret)

                with patch.dict(mount.__opts__, {"test": False}):
                    comt = (
                        "Swap {0} already active. swap not present"
                        " in /etc/filesystems on AIX.".format(name)
                    )
                    ret.update({"comment": comt, "result": False})
                    self.assertDictEqual(mount.swap(name), ret)

                    with patch.dict(
                        mount.__salt__,
                        {"mount.filesystems": mock_emt, "mount.set_filesystems": mock},
                    ):
                        comt = (
                            "Swap {0} already active. swap not present"
                            " in /etc/filesystems on AIX.".format(name)
                        )
                        ret.update({"comment": comt, "result": False})
                        self.assertDictEqual(mount.swap(name), ret)

    # 'unmounted' function tests: 1

    def test_unmounted(self):
        """
        Test to verify that a device is not mounted
        """
        name = "/mnt/sdb"
        device = "/dev/sdb5"

        ret = {"name": name, "result": None, "comment": "", "changes": {}}

        mock_f = MagicMock(return_value=False)
        mock_t = MagicMock(return_value=True)
        mock_dev = MagicMock(return_value={name: {"device": device}})
        mock_fs = MagicMock(return_value={name: {"device": name}})
        mock_mnt = MagicMock(side_effect=[{name: {}}, {}, {}, {}])

        name3 = os.path.realpath("/mnt/jfs2")
        device3 = "/dev/hd1"
        fstype3 = "jfs2"
        opts3 = [""]
        mock_mnta = MagicMock(return_value={name3: {"device": device3, "opts": opts3}})
        mock_aixfs = MagicMock(return_value={name: {"dev": name3, "fstype": fstype3}})
        mock_delete_cache = MagicMock(return_value=True)

        comt3 = (
            "Mount point /mnt/sdb is unmounted but needs to be purged "
            "from /etc/auto_salt to be made persistent"
        )

        with patch.dict(mount.__grains__, {"os": "Darwin"}):
            with patch.dict(
                mount.__salt__,
                {
                    "mount.active": mock_mnt,
                    "mount.automaster": mock_fs,
                    "file.is_link": mock_f,
                },
            ):
                with patch.dict(mount.__opts__, {"test": True}):
                    comt = "Mount point {0} is mounted but should not " "be".format(
                        name
                    )
                    ret.update({"comment": comt})
                    self.assertDictEqual(mount.unmounted(name, device), ret)

                    comt = (
                        "Target was already unmounted. "
                        "fstab entry for device {0} not found".format(device)
                    )
                    ret.update({"comment": comt, "result": True})
                    self.assertDictEqual(
                        mount.unmounted(name, device, persist=True), ret
                    )

                    with patch.dict(mount.__salt__, {"mount.automaster": mock_dev}):
                        ret.update({"comment": comt3, "result": None})
                        self.assertDictEqual(
                            mount.unmounted(name, device, persist=True), ret
                        )

                    comt = "Target was already unmounted"
                    ret.update({"comment": comt, "result": True})
                    self.assertDictEqual(mount.unmounted(name, device), ret)

        with patch.dict(mount.__grains__, {"os": "AIX"}):
            with patch.dict(
                mount.__salt__,
                {
                    "mount.active": mock_mnta,
                    "mount.filesystems": mock_aixfs,
                    "file.is_link": mock_f,
                },
            ):
                with patch.dict(mount.__opts__, {"test": True}):
                    comt = "Target was already unmounted"
                    ret.update({"comment": comt, "result": True})
                    self.assertDictEqual(mount.unmounted(name, device), ret)

                    comt = (
                        "Target was already unmounted. "
                        "fstab entry for device /dev/sdb5 not found"
                    )
                    ret.update({"comment": comt, "result": True})
                    self.assertDictEqual(
                        mount.unmounted(name, device, persist=True), ret
                    )

                    with patch.dict(mount.__salt__, {"mount.filesystems": mock_dev}):
                        comt = "Mount point {0} is mounted but should not " "be".format(
                            name3
                        )
                        ret.update({"comment": comt, "result": None, "name": name3})
                        self.assertDictEqual(
                            mount.unmounted(name3, device3, persist=True), ret
                        )

                        with patch.dict(mount.__opts__, {"test": False}), patch.dict(
                            mount.__salt__,
                            {
                                "mount.umount": mock_t,
                                "mount.delete_mount_cache": mock_delete_cache,
                            },
                        ):
                            comt = "Target was successfully unmounted"
                            ret.update(
                                {
                                    "comment": comt,
                                    "result": True,
                                    "name": name3,
                                    "changes": {"umount": True},
                                }
                            )
                            self.assertDictEqual(mount.unmounted(name3, device3), ret)

    # 'mod_watch' function tests: 1

    def test_mod_watch(self):
        """
        Test the mounted watcher, called to invoke the watch command.
        """
        name = "/mnt/sdb"

        ret = {
            "name": name,
            "result": True,
            "comment": "Watch not supported in unmount at this time",
            "changes": {},
        }

        self.assertDictEqual(mount.mod_watch(name, sfun="unmount"), ret)

    def test_mounted_multiple_mounts(self):
        """
        Test to verify that a device is mounted.
        """
        name = "/mnt/nfs1"
        device = "localhost:/mnt/nfsshare"
        fstype = "nfs4"

        name2 = "/mnt/nfs2"
        device2 = "localhost:/mnt/nfsshare"
        fstype2 = "nfs4"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        mock = MagicMock(
            side_effect=[
                "new",
                "present",
                "new",
                "change",
                "bad config",
                "salt",
                "present",
            ]
        )
        mock_t = MagicMock(return_value=True)
        mock_f = MagicMock(return_value=False)
        mock_ret = MagicMock(return_value={"retcode": 1})
        mock_mnt = MagicMock(
            return_value={name: {"device": device, "opts": [], "superopts": []}}
        )
        mock_read_cache = MagicMock(return_value={})
        mock_write_cache = MagicMock(return_value=True)
        mock_user = MagicMock(return_value={"uid": 510})
        mock_group = MagicMock(return_value={"gid": 100})
        mock_str = MagicMock(return_value="salt")
        mock_fstab_config = ["localhost:/mnt/nfsshare		/mnt/nfs1	nfs	defaults	0 0"]

        # Test no change for uid provided as a name #25293
        with patch.dict(mount.__grains__, {"os": "CentOS"}):
            with patch.dict(mount.__opts__, {"test": True}):
                with patch.dict(
                    mount.__salt__,
                    {
                        "mount.active": mock_mnt,
                        "mount.mount": mock_str,
                        "mount.umount": mock_f,
                        "mount.read_mount_cache": mock_read_cache,
                        "mount.write_mount_cache": mock_write_cache,
                        "user.info": mock_user,
                        "group.info": mock_group,
                    },
                ):
                    with patch.object(os.path, "exists", mock_t):
                        comt = "/mnt/nfs2 would be mounted"
                        ret.update({"name": name2, "result": None})
                        ret.update({"comment": comt, "changes": {}})
                        self.assertDictEqual(
                            mount.mounted(name2, device2, fstype2, opts=[]), ret
                        )

    def test__convert_to_fast_none(self):
        """
        Test the device name conversor
        """
        assert mount._convert_to("/dev/sda1", None) == "/dev/sda1"

    def test__convert_to_fast_device(self):
        """
        Test the device name conversor
        """
        assert mount._convert_to("/dev/sda1", "device") == "/dev/sda1"

    def test__convert_to_fast_token(self):
        """
        Test the device name conversor
        """
        assert mount._convert_to("LABEL=home", "label") == "LABEL=home"

    def test__convert_to_device_none(self):
        """
        Test the device name conversor
        """
        salt_mock = {
            "disk.blkid": MagicMock(return_value={}),
        }
        with patch.dict(mount.__salt__, salt_mock):
            assert mount._convert_to("/dev/sda1", "uuid") is None
            salt_mock["disk.blkid"].assert_called_with("/dev/sda1")

    def test__convert_to_device_token(self):
        """
        Test the device name conversor
        """
        uuid = "988c663d-74a2-432b-ba52-3eea34015f22"
        salt_mock = {
            "disk.blkid": MagicMock(return_value={"/dev/sda1": {"UUID": uuid}}),
        }
        with patch.dict(mount.__salt__, salt_mock):
            uuid = "UUID={}".format(uuid)
            assert mount._convert_to("/dev/sda1", "uuid") == uuid
            salt_mock["disk.blkid"].assert_called_with("/dev/sda1")

    def test__convert_to_token_device(self):
        """
        Test the device name conversor
        """
        uuid = "988c663d-74a2-432b-ba52-3eea34015f22"
        salt_mock = {
            "disk.blkid": MagicMock(return_value={"/dev/sda1": {"UUID": uuid}}),
        }
        with patch.dict(mount.__salt__, salt_mock):
            uuid = "UUID={}".format(uuid)
            assert mount._convert_to(uuid, "device") == "/dev/sda1"
            salt_mock["disk.blkid"].assert_called_with(token=uuid)

    def test_fstab_present_macos_test_present(self):
        """
        Test fstab_present
        """
        ret = {
            "name": "/dev/sda1",
            "result": None,
            "changes": {},
            "comment": ["/home entry is already in /etc/auto_salt."],
        }

        grains_mock = {"os": "MacOS"}
        opts_mock = {"test": True}
        salt_mock = {"mount.set_automaster": MagicMock(return_value="present")}
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_present("/dev/sda1", "/home", "ext2") == ret
            salt_mock["mount.set_automaster"].assert_called_with(
                name="/home",
                device="/dev/sda1",
                fstype="ext2",
                opts="noowners",
                config="/etc/auto_salt",
                test=True,
                not_change=False,
            )

    def test_fstab_present_aix_test_present(self):
        """
        Test fstab_present
        """
        ret = {
            "name": "/dev/sda1",
            "result": None,
            "changes": {},
            "comment": ["/home entry is already in /etc/filesystems."],
        }

        grains_mock = {"os": "AIX"}
        opts_mock = {"test": True}
        salt_mock = {"mount.set_filesystems": MagicMock(return_value="present")}
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_present("/dev/sda1", "/home", "ext2") == ret
            salt_mock["mount.set_filesystems"].assert_called_with(
                name="/home",
                device="/dev/sda1",
                fstype="ext2",
                mount=True,
                opts="",
                config="/etc/filesystems",
                test=True,
                match_on="auto",
                not_change=False,
            )

    def test_fstab_present_test_present(self):
        """
        Test fstab_present
        """
        ret = {
            "name": "/dev/sda1",
            "result": None,
            "changes": {},
            "comment": ["/home entry is already in /etc/fstab."],
        }

        grains_mock = {"os": "Linux"}
        opts_mock = {"test": True}
        salt_mock = {"mount.set_fstab": MagicMock(return_value="present")}
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_present("/dev/sda1", "/home", "ext2") == ret
            salt_mock["mount.set_fstab"].assert_called_with(
                name="/home",
                device="/dev/sda1",
                fstype="ext2",
                opts="defaults",
                dump=0,
                pass_num=0,
                config="/etc/fstab",
                test=True,
                match_on="auto",
                not_change=False,
            )

    def test_fstab_present_test_new(self):
        """
        Test fstab_present
        """
        ret = {
            "name": "/dev/sda1",
            "result": None,
            "changes": {},
            "comment": ["/home entry will be written in /etc/fstab."],
        }

        grains_mock = {"os": "Linux"}
        opts_mock = {"test": True}
        salt_mock = {"mount.set_fstab": MagicMock(return_value="new")}
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_present("/dev/sda1", "/home", "ext2") == ret
            salt_mock["mount.set_fstab"].assert_called_with(
                name="/home",
                device="/dev/sda1",
                fstype="ext2",
                opts="defaults",
                dump=0,
                pass_num=0,
                config="/etc/fstab",
                test=True,
                match_on="auto",
                not_change=False,
            )

    def test_fstab_present_test_change(self):
        """
        Test fstab_present
        """
        ret = {
            "name": "/dev/sda1",
            "result": None,
            "changes": {},
            "comment": ["/home entry will be updated in /etc/fstab."],
        }

        grains_mock = {"os": "Linux"}
        opts_mock = {"test": True}
        salt_mock = {"mount.set_fstab": MagicMock(return_value="change")}
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_present("/dev/sda1", "/home", "ext2") == ret
            salt_mock["mount.set_fstab"].assert_called_with(
                name="/home",
                device="/dev/sda1",
                fstype="ext2",
                opts="defaults",
                dump=0,
                pass_num=0,
                config="/etc/fstab",
                test=True,
                match_on="auto",
                not_change=False,
            )

    def test_fstab_present_test_error(self):
        """
        Test fstab_present
        """
        ret = {
            "name": "/dev/sda1",
            "result": False,
            "changes": {},
            "comment": ["/home entry cannot be created in /etc/fstab: error."],
        }

        grains_mock = {"os": "Linux"}
        opts_mock = {"test": True}
        salt_mock = {"mount.set_fstab": MagicMock(return_value="error")}
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_present("/dev/sda1", "/home", "ext2") == ret
            salt_mock["mount.set_fstab"].assert_called_with(
                name="/home",
                device="/dev/sda1",
                fstype="ext2",
                opts="defaults",
                dump=0,
                pass_num=0,
                config="/etc/fstab",
                test=True,
                match_on="auto",
                not_change=False,
            )

    def test_fstab_present_macos_present(self):
        """
        Test fstab_present
        """
        ret = {
            "name": "/dev/sda1",
            "result": True,
            "changes": {},
            "comment": ["/home entry was already in /etc/auto_salt."],
        }

        grains_mock = {"os": "MacOS"}
        opts_mock = {"test": False}
        salt_mock = {"mount.set_automaster": MagicMock(return_value="present")}
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_present("/dev/sda1", "/home", "ext2") == ret
            salt_mock["mount.set_automaster"].assert_called_with(
                name="/home",
                device="/dev/sda1",
                fstype="ext2",
                opts="noowners",
                config="/etc/auto_salt",
                not_change=False,
            )

    def test_fstab_present_aix_present(self):
        """
        Test fstab_present
        """
        ret = {
            "name": "/dev/sda1",
            "result": True,
            "changes": {},
            "comment": ["/home entry was already in /etc/filesystems."],
        }

        grains_mock = {"os": "AIX"}
        opts_mock = {"test": False}
        salt_mock = {"mount.set_filesystems": MagicMock(return_value="present")}
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_present("/dev/sda1", "/home", "ext2") == ret
            salt_mock["mount.set_filesystems"].assert_called_with(
                name="/home",
                device="/dev/sda1",
                fstype="ext2",
                mount=True,
                opts="",
                config="/etc/filesystems",
                match_on="auto",
                not_change=False,
            )

    def test_fstab_present_present(self):
        """
        Test fstab_present
        """
        ret = {
            "name": "/dev/sda1",
            "result": True,
            "changes": {},
            "comment": ["/home entry was already in /etc/fstab."],
        }

        grains_mock = {"os": "Linux"}
        opts_mock = {"test": False}
        salt_mock = {"mount.set_fstab": MagicMock(return_value="present")}
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_present("/dev/sda1", "/home", "ext2") == ret
            salt_mock["mount.set_fstab"].assert_called_with(
                name="/home",
                device="/dev/sda1",
                fstype="ext2",
                opts="defaults",
                dump=0,
                pass_num=0,
                config="/etc/fstab",
                match_on="auto",
                not_change=False,
            )

    def test_fstab_present_new(self):
        """
        Test fstab_present
        """
        ret = {
            "name": "/dev/sda1",
            "result": True,
            "changes": {"persist": "new"},
            "comment": ["/home entry added in /etc/fstab."],
        }

        grains_mock = {"os": "Linux"}
        opts_mock = {"test": False}
        salt_mock = {"mount.set_fstab": MagicMock(return_value="new")}
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_present("/dev/sda1", "/home", "ext2") == ret
            salt_mock["mount.set_fstab"].assert_called_with(
                name="/home",
                device="/dev/sda1",
                fstype="ext2",
                opts="defaults",
                dump=0,
                pass_num=0,
                config="/etc/fstab",
                match_on="auto",
                not_change=False,
            )

    def test_fstab_present_change(self):
        """
        Test fstab_present
        """
        ret = {
            "name": "/dev/sda1",
            "result": True,
            "changes": {"persist": "change"},
            "comment": ["/home entry updated in /etc/fstab."],
        }

        grains_mock = {"os": "Linux"}
        opts_mock = {"test": False}
        salt_mock = {"mount.set_fstab": MagicMock(return_value="change")}
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_present("/dev/sda1", "/home", "ext2") == ret
            salt_mock["mount.set_fstab"].assert_called_with(
                name="/home",
                device="/dev/sda1",
                fstype="ext2",
                opts="defaults",
                dump=0,
                pass_num=0,
                config="/etc/fstab",
                match_on="auto",
                not_change=False,
            )

    def test_fstab_present_fail(self):
        """
        Test fstab_present
        """
        ret = {
            "name": "/dev/sda1",
            "result": False,
            "changes": {},
            "comment": ["/home entry cannot be changed in /etc/fstab: error."],
        }

        grains_mock = {"os": "Linux"}
        opts_mock = {"test": False}
        salt_mock = {"mount.set_fstab": MagicMock(return_value="error")}
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_present("/dev/sda1", "/home", "ext2") == ret
            salt_mock["mount.set_fstab"].assert_called_with(
                name="/home",
                device="/dev/sda1",
                fstype="ext2",
                opts="defaults",
                dump=0,
                pass_num=0,
                config="/etc/fstab",
                match_on="auto",
                not_change=False,
            )

    def test_fstab_absent_macos_test_absent(self):
        """
        Test fstab_absent
        """
        ret = {
            "name": "/dev/sda1",
            "result": None,
            "changes": {},
            "comment": ["/home entry is already missing in /etc/auto_salt."],
        }

        grains_mock = {"os": "MacOS"}
        opts_mock = {"test": True}
        salt_mock = {"mount.automaster": MagicMock(return_value={})}
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_absent("/dev/sda1", "/home") == ret
            salt_mock["mount.automaster"].assert_called_with("/etc/auto_salt")

    def test_fstab_absent_aix_test_absent(self):
        """
        Test fstab_absent
        """
        ret = {
            "name": "/dev/sda1",
            "result": None,
            "changes": {},
            "comment": ["/home entry is already missing in /etc/filesystems."],
        }

        grains_mock = {"os": "AIX"}
        opts_mock = {"test": True}
        salt_mock = {"mount.filesystems": MagicMock(return_value={})}
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_absent("/dev/sda1", "/home") == ret
            salt_mock["mount.filesystems"].assert_called_with("/etc/filesystems")

    def test_fstab_absent_test_absent(self):
        """
        Test fstab_absent
        """
        ret = {
            "name": "/dev/sda1",
            "result": None,
            "changes": {},
            "comment": ["/home entry is already missing in /etc/fstab."],
        }

        grains_mock = {"os": "Linux"}
        opts_mock = {"test": True}
        salt_mock = {"mount.fstab": MagicMock(return_value={})}
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_absent("/dev/sda1", "/home") == ret
            salt_mock["mount.fstab"].assert_called_with("/etc/fstab")

    def test_fstab_absent_test_present(self):
        """
        Test fstab_absent
        """
        ret = {
            "name": "/dev/sda1",
            "result": None,
            "changes": {},
            "comment": ["/home entry will be removed from /etc/fstab."],
        }

        grains_mock = {"os": "Linux"}
        opts_mock = {"test": True}
        salt_mock = {"mount.fstab": MagicMock(return_value={"/home": {}})}
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_absent("/dev/sda1", "/home") == ret
            salt_mock["mount.fstab"].assert_called_with("/etc/fstab")

    def test_fstab_absent_macos_present(self):
        """
        Test fstab_absent
        """
        ret = {
            "name": "/dev/sda1",
            "result": True,
            "changes": {"persist": "removed"},
            "comment": ["/home entry removed from /etc/auto_salt."],
        }

        grains_mock = {"os": "MacOS"}
        opts_mock = {"test": False}
        salt_mock = {
            "mount.automaster": MagicMock(return_value={"/home": {}}),
            "mount.rm_automaster": MagicMock(return_value=True),
        }
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_absent("/dev/sda1", "/home") == ret
            salt_mock["mount.automaster"].assert_called_with("/etc/auto_salt")
            salt_mock["mount.rm_automaster"].assert_called_with(
                name="/home", device="/dev/sda1", config="/etc/auto_salt"
            )

    def test_fstab_absent_aix_present(self):
        """
        Test fstab_absent
        """
        ret = {
            "name": "/dev/sda1",
            "result": True,
            "changes": {"persist": "removed"},
            "comment": ["/home entry removed from /etc/filesystems."],
        }

        grains_mock = {"os": "AIX"}
        opts_mock = {"test": False}
        salt_mock = {
            "mount.filesystems": MagicMock(return_value={"/home": {}}),
            "mount.rm_filesystems": MagicMock(return_value=True),
        }
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_absent("/dev/sda1", "/home") == ret
            salt_mock["mount.filesystems"].assert_called_with("/etc/filesystems")
            salt_mock["mount.rm_filesystems"].assert_called_with(
                name="/home", device="/dev/sda1", config="/etc/filesystems"
            )

    def test_fstab_absent_present(self):
        """
        Test fstab_absent
        """
        ret = {
            "name": "/dev/sda1",
            "result": True,
            "changes": {"persist": "removed"},
            "comment": ["/home entry removed from /etc/fstab."],
        }

        grains_mock = {"os": "Linux"}
        opts_mock = {"test": False}
        salt_mock = {
            "mount.fstab": MagicMock(return_value={"/home": {}}),
            "mount.rm_fstab": MagicMock(return_value=True),
        }
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_absent("/dev/sda1", "/home") == ret
            salt_mock["mount.fstab"].assert_called_with("/etc/fstab")
            salt_mock["mount.rm_fstab"].assert_called_with(
                name="/home", device="/dev/sda1", config="/etc/fstab"
            )

    def test_fstab_absent_absent(self):
        """
        Test fstab_absent
        """
        ret = {
            "name": "/dev/sda1",
            "result": True,
            "changes": {},
            "comment": ["/home entry is already missing in /etc/fstab."],
        }

        grains_mock = {"os": "Linux"}
        opts_mock = {"test": False}
        salt_mock = {"mount.fstab": MagicMock(return_value={})}
        with patch.dict(mount.__grains__, grains_mock), patch.dict(
            mount.__opts__, opts_mock
        ), patch.dict(mount.__salt__, salt_mock):
            assert mount.fstab_absent("/dev/sda1", "/home") == ret
            salt_mock["mount.fstab"].assert_called_with("/etc/fstab")



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.win_ntp as win_ntp

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class WinNtpTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.win_ntp
    """

    def setup_loader_modules(self):
        return {win_ntp: {}}

    # 'set_servers' function tests: 1

    def test_set_servers(self):
        """
        Test if it set Windows to use a list of NTP servers
        """
        # Windows Time (W32Time) service is not started
        # Windows Time (W32Time) service fails to start
        mock_service = MagicMock(return_value=False)
        with patch.dict(
            win_ntp.__salt__,
            {"service.status": mock_service, "service.start": mock_service},
        ):
            self.assertFalse(win_ntp.set_servers("pool.ntp.org"))

        # Windows Time service is running
        # Fail to set NTP servers
        mock_service = MagicMock(return_value=True)
        mock_cmd = MagicMock(
            side_effect=[
                "Failure",
                "Failure",
                "Failure",
                "NtpServer: time.windows.com,0x8",
            ]
        )
        with patch.dict(
            win_ntp.__salt__, {"service.status": mock_service, "cmd.run": mock_cmd}
        ):
            self.assertFalse(win_ntp.set_servers("pool.ntp.org"))

        # Windows Time service is running
        # Successfully set NTP servers
        mock_cmd = MagicMock(
            side_effect=["Success", "Success", "Success", "NtpServer: pool.ntp.org"]
        )
        with patch.dict(
            win_ntp.__salt__,
            {
                "service.status": mock_service,
                "service.restart": mock_service,
                "cmd.run": mock_cmd,
            },
        ):
            self.assertTrue(win_ntp.set_servers("pool.ntp.org"))

    # 'get_servers' function tests: 1

    def test_get_servers(self):
        """
        Test if it get list of configured NTP servers
        """
        mock_cmd = MagicMock(side_effect=["", "NtpServer: SALT", "NtpServer"])
        with patch.dict(win_ntp.__salt__, {"cmd.run": mock_cmd}):
            self.assertFalse(win_ntp.get_servers())

            self.assertListEqual(win_ntp.get_servers(), ["SALT"])

            self.assertFalse(win_ntp.get_servers())



# -*- coding: utf-8 -*-
"""
    :codeauthor: :email:`Anthony Shaw <anthonyshaw@apache.org>`
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

import tests.support.napalm as napalm_test_support

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock
from tests.support.unit import TestCase

import salt.modules.napalm_netacl as napalm_acl  # NOQA


def mock_capirca_term_config(platform, filter_name, term_name, *args, **kwargs):
    assert platform == "cisco"
    assert filter_name == "test_filter"
    assert term_name == "test_term"
    return "test_config"


def mock_capirca_filter_config(platform, filter_name, *args, **kwargs):
    assert platform == "cisco"
    assert filter_name == "test_filter"
    return "test_config"


def mock_capirca_policy_config(platform, *args, **kwargs):
    assert platform == "cisco"
    return "test_config"


def mock_net_load_config(text, *args, **kwargs):
    assert text == "test_config"
    return napalm_test_support.TEST_TERM_CONFIG


def mock_capirca_get_filter_pillar(filter_, *args, **kwargs):
    assert filter_ == "test_filter"
    return {"test": "value"}


def mock_capirca_get_term_pillar(filter_, term, *args, **kwargs):
    assert filter_ == "test_filter"
    assert term == "test_term"
    return {"test": "value"}


class NapalmAclModuleTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        module_globals = {
            "__salt__": {
                "config.option": MagicMock(
                    return_value={"test": {"driver": "test", "key": "2orgk34kgk34g"}}
                ),
                "file.file_exists": napalm_test_support.true,
                "file.join": napalm_test_support.join,
                "file.get_managed": napalm_test_support.get_managed_file,
                "random.hash": napalm_test_support.random_hash,
                "capirca.get_term_config": mock_capirca_term_config,
                "capirca.get_policy_config": mock_capirca_policy_config,
                "capirca.get_filter_config": mock_capirca_filter_config,
                "capirca.get_filter_pillar": mock_capirca_get_filter_pillar,
                "capirca.get_term_pillar": mock_capirca_get_term_pillar,
                "net.load_config": mock_net_load_config,
            },
            "__grains__": {"os": "ios", "vendor": "cisco", "model": "3750X"},
        }

        return {napalm_acl: module_globals}

    def test_load_term_config(self):
        ret = napalm_acl.load_term_config("test_filter", "test_term")
        assert ret["already_configured"] is False

    def test_load_filter_config(self):
        ret = napalm_acl.load_filter_config("test_filter", "test_term")
        assert ret["already_configured"] is False

    def test_load_policy_config(self):
        ret = napalm_acl.load_policy_config("test_filter", "test_term")
        assert ret["already_configured"] is False

    def test_get_filter_pillar(self):
        ret = napalm_acl.get_filter_pillar("test_filter")
        assert ret["test"] == "value"

    def test_get_term_pillar(self):
        ret = napalm_acl.get_term_pillar("test_filter", "test_term")
        assert ret["test"] == "value"



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import os

# Import Salt Libs
import salt.states.ssh_known_hosts as ssh_known_hosts

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class SshKnownHostsTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.ssh_known_hosts
    """

    def setup_loader_modules(self):
        return {ssh_known_hosts: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to verifies that the specified host is known by the specified user.
        """
        name = "github.com"
        user = "root"
        key = "16:27:ac:a5:76:28:2d:36:63:1b:56:4d:eb:df:a6:48"
        fingerprint = [key]

        ret = {"name": name, "changes": {}, "result": False, "comment": ""}

        with patch.dict(ssh_known_hosts.__opts__, {"test": True}):
            with patch.object(os.path, "isabs", MagicMock(return_value=False)):
                comt = 'If not specifying a "user", ' 'specify an absolute "config".'
                ret.update({"comment": comt})
                self.assertDictEqual(ssh_known_hosts.present(name), ret)

            comt = 'Specify either "key" or "fingerprint", not both.'
            ret.update({"comment": comt})
            self.assertDictEqual(
                ssh_known_hosts.present(name, user, key=key, fingerprint=[key]), ret
            )

            comt = 'Required argument "enc" if using "key" argument.'
            ret.update({"comment": comt})
            self.assertDictEqual(ssh_known_hosts.present(name, user, key=key), ret)

            mock = MagicMock(side_effect=["exists", "add", "update"])
            with patch.dict(ssh_known_hosts.__salt__, {"ssh.check_known_host": mock}):
                comt = "Host github.com is already in .ssh/known_hosts"
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(ssh_known_hosts.present(name, user), ret)

                comt = "Key for github.com is set to be" " added to .ssh/known_hosts"
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(ssh_known_hosts.present(name, user), ret)

                comt = "Key for github.com is set to be " "updated in .ssh/known_hosts"
                ret.update({"comment": comt})
                self.assertDictEqual(ssh_known_hosts.present(name, user), ret)

        with patch.dict(ssh_known_hosts.__opts__, {"test": False}):
            result = {"status": "exists", "error": ""}
            mock = MagicMock(return_value=result)
            with patch.dict(ssh_known_hosts.__salt__, {"ssh.set_known_host": mock}):
                comt = "github.com already exists in .ssh/known_hosts"
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(ssh_known_hosts.present(name, user), ret)

            result = {"status": "error", "error": ""}
            mock = MagicMock(return_value=result)
            with patch.dict(ssh_known_hosts.__salt__, {"ssh.set_known_host": mock}):
                ret.update({"comment": "", "result": False})
                self.assertDictEqual(ssh_known_hosts.present(name, user), ret)

            result = {
                "status": "updated",
                "error": "",
                "new": [{"fingerprint": fingerprint, "key": key}],
                "old": "",
            }
            mock = MagicMock(return_value=result)
            with patch.dict(ssh_known_hosts.__salt__, {"ssh.set_known_host": mock}):
                comt = "{0}'s key saved to .ssh/known_hosts (key: {1})".format(
                    name, key
                )
                ret.update(
                    {
                        "comment": comt,
                        "result": True,
                        "changes": {
                            "new": [{"fingerprint": fingerprint, "key": key}],
                            "old": "",
                        },
                    }
                )
                self.assertDictEqual(ssh_known_hosts.present(name, user, key=key), ret)

                comt = "{0}'s key saved to .ssh/known_hosts (fingerprint: {1})".format(
                    name, fingerprint
                )
                ret.update({"comment": comt})
                self.assertDictEqual(ssh_known_hosts.present(name, user), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to verifies that the specified host is not known by the given user.
        """
        name = "github.com"
        user = "root"

        ret = {"name": name, "changes": {}, "result": False, "comment": ""}

        with patch.object(os.path, "isabs", MagicMock(return_value=False)):
            comt = 'If not specifying a "user", ' 'specify an absolute "config".'
            ret.update({"comment": comt})
            self.assertDictEqual(ssh_known_hosts.absent(name), ret)

        mock = MagicMock(return_value=False)
        with patch.dict(ssh_known_hosts.__salt__, {"ssh.get_known_host_entries": mock}):
            comt = "Host is already absent"
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(ssh_known_hosts.absent(name, user), ret)

        mock = MagicMock(return_value=True)
        with patch.dict(ssh_known_hosts.__salt__, {"ssh.get_known_host_entries": mock}):
            with patch.dict(ssh_known_hosts.__opts__, {"test": True}):
                comt = (
                    "Key for github.com is set to be" " removed from .ssh/known_hosts"
                )
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(ssh_known_hosts.absent(name, user), ret)

            with patch.dict(ssh_known_hosts.__opts__, {"test": False}):
                result = {"status": "error", "error": ""}
                mock = MagicMock(return_value=result)
                with patch.dict(ssh_known_hosts.__salt__, {"ssh.rm_known_host": mock}):
                    ret.update({"comment": "", "result": False})
                    self.assertDictEqual(ssh_known_hosts.absent(name, user), ret)

                result = {"status": "removed", "error": "", "comment": "removed"}
                mock = MagicMock(return_value=result)
                with patch.dict(ssh_known_hosts.__salt__, {"ssh.rm_known_host": mock}):
                    ret.update(
                        {
                            "comment": "removed",
                            "result": True,
                            "changes": {"new": None, "old": True},
                        }
                    )
                    self.assertDictEqual(ssh_known_hosts.absent(name, user), ret)



# -*- coding: utf-8 -*-
"""
Tests for the Telegram execution module.

:codeauthor: :email:`Roald Nefs (info@roaldnefs.com)`
"""

# Import Python Libs
from __future__ import absolute_import

import logging

# Import Salt Libs
import salt.modules.telegram as telegram

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, Mock
from tests.support.unit import TestCase

log = logging.getLogger(__name__)


class RequestMock(Mock):
    """
    Request Mock
    """

    def get(self, *args, **kwargs):
        return RequestResponseMock()

    def put(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        return RequestPutResponseMock()

    def delete(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        return RequestResponseMock()


class RequestResponseMock(Mock):
    """
    Request Response Mock
    """

    def json(self):
        return [
            {"url": "http://example.org", "_id": 1234},
        ]


class RequestPutResponseMock(Mock):
    """
    Request Put Response Mock
    """

    ok = True

    def json(self):
        return {"_id": 4321}


class TelegramModuleTest(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.telegram.
    """

    def setup_loader_modules(self):
        module_globals = {
            "__salt__": {
                "config.get": MagicMock(
                    return_value={
                        "telegram": {
                            "chat_id": "123456789",
                            "token": "000000000:xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                        }
                    }
                ),
                "requests.put": Mock(),
            },
            "requests": RequestMock(),
        }
        return {telegram: module_globals}

    def test_post_message(self):
        """
        Test the post_message function.
        """
        message = "Hello World!"
        self.assertTrue(telegram.post_message(message))



# -*- coding: utf-8 -*-
"""
A convenience system to manage jobs, both active and already run
"""

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import fnmatch
import logging
import os

# Import salt libs
import salt.client
import salt.minion
import salt.payload
import salt.returners
import salt.utils.args
import salt.utils.files
import salt.utils.jid
import salt.utils.master
from salt.exceptions import SaltClientError

# Import 3rd-party libs
from salt.ext import six

try:
    import dateutil.parser as dateutil_parser

    DATEUTIL_SUPPORT = True
except ImportError:
    DATEUTIL_SUPPORT = False

log = logging.getLogger(__name__)


def master():
    """
    Return the actively executing runners for the master

    CLI Example:

    .. code-block:: bash

        salt-run jobs.master
    """
    return salt.utils.master.get_running_jobs(__opts__)


def active(display_progress=False):
    """
    Return a report on all actively running jobs from a job id centric
    perspective

    CLI Example:

    .. code-block:: bash

        salt-run jobs.active
    """
    ret = {}
    client = salt.client.get_local_client(__opts__["conf_file"])
    try:
        active_ = client.cmd("*", "saltutil.running", timeout=__opts__["timeout"])
    except SaltClientError as client_error:
        print(client_error)
        return ret

    if display_progress:
        __jid_event__.fire_event(
            {
                "message": "Attempting to contact minions: {0}".format(
                    list(active_.keys())
                )
            },
            "progress",
        )
    for minion, data in six.iteritems(active_):
        if display_progress:
            __jid_event__.fire_event(
                {"message": "Received reply from minion {0}".format(minion)}, "progress"
            )
        if not isinstance(data, list):
            continue
        for job in data:
            if not job["jid"] in ret:
                ret[job["jid"]] = _format_jid_instance(job["jid"], job)
                ret[job["jid"]].update(
                    {"Running": [{minion: job.get("pid", None)}], "Returned": []}
                )
            else:
                ret[job["jid"]]["Running"].append({minion: job["pid"]})

    mminion = salt.minion.MasterMinion(__opts__)
    for jid in ret:
        returner = _get_returner(
            (__opts__["ext_job_cache"], __opts__["master_job_cache"])
        )
        data = mminion.returners["{0}.get_jid".format(returner)](jid)
        if data:
            for minion in data:
                if minion not in ret[jid]["Returned"]:
                    ret[jid]["Returned"].append(minion)

    return ret


def lookup_jid(
    jid, ext_source=None, returned=True, missing=False, display_progress=False
):
    """
    Return the printout from a previously executed job

    jid
        The jid to look up.

    ext_source
        The external job cache to use. Default: `None`.

    returned : True
        If ``True``, include the minions that did return from the command.

        .. versionadded:: 2015.8.0

    missing : False
        If ``True``, include the minions that did *not* return from the
        command.

    display_progress : False
        If ``True``, fire progress events.

        .. versionadded:: 2015.5.0

    CLI Example:

    .. code-block:: bash

        salt-run jobs.lookup_jid 20130916125524463507
        salt-run jobs.lookup_jid 20130916125524463507 --out=highstate
    """
    ret = {}
    mminion = salt.minion.MasterMinion(__opts__)
    returner = _get_returner(
        (__opts__["ext_job_cache"], ext_source, __opts__["master_job_cache"])
    )

    try:
        data = list_job(jid, ext_source=ext_source, display_progress=display_progress)
    except TypeError:
        return "Requested returner could not be loaded. " "No JIDs could be retrieved."

    targeted_minions = data.get("Minions", [])
    returns = data.get("Result", {})

    if returns:
        for minion in returns:
            if display_progress:
                __jid_event__.fire_event({"message": minion}, "progress")
            if "return" in returns[minion]:
                if returned:
                    ret[minion] = returns[minion].get("return")
            else:
                if returned:
                    ret[minion] = returns[minion].get("return")
    if missing:
        for minion_id in (x for x in targeted_minions if x not in returns):
            ret[minion_id] = "Minion did not return"

    # We need to check to see if the 'out' key is present and use it to specify
    # the correct outputter, so we get highstate output for highstate runs.
    try:
        # Check if the return data has an 'out' key. We'll use that as the
        # outputter in the absence of one being passed on the CLI.
        outputter = returns[next(iter(returns))].get("out")
    except (StopIteration, AttributeError):
        outputter = None

    if outputter:
        return {"outputter": outputter, "data": ret}
    else:
        return ret


def list_job(jid, ext_source=None, display_progress=False):
    """
    List a specific job given by its jid

    ext_source
        If provided, specifies which external job cache to use.

    display_progress : False
        If ``True``, fire progress events.

        .. versionadded:: 2015.8.8

    CLI Example:

    .. code-block:: bash

        salt-run jobs.list_job 20130916125524463507
        salt-run jobs.list_job 20130916125524463507 --out=pprint
    """
    ret = {"jid": jid}
    mminion = salt.minion.MasterMinion(__opts__)
    returner = _get_returner(
        (__opts__["ext_job_cache"], ext_source, __opts__["master_job_cache"])
    )
    if display_progress:
        __jid_event__.fire_event(
            {"message": "Querying returner: {0}".format(returner)}, "progress"
        )

    job = mminion.returners["{0}.get_load".format(returner)](jid)
    ret.update(_format_jid_instance(jid, job))
    ret["Result"] = mminion.returners["{0}.get_jid".format(returner)](jid)

    fstr = "{0}.get_endtime".format(__opts__["master_job_cache"])
    if __opts__.get("job_cache_store_endtime") and fstr in mminion.returners:
        endtime = mminion.returners[fstr](jid)
        if endtime:
            ret["EndTime"] = endtime

    return ret


def list_jobs(
    ext_source=None,
    outputter=None,
    search_metadata=None,
    search_function=None,
    search_target=None,
    start_time=None,
    end_time=None,
    display_progress=False,
):
    """
    List all detectable jobs and associated functions

    ext_source
        If provided, specifies which external job cache to use.

    **FILTER OPTIONS**

    .. note::
        If more than one of the below options are used, only jobs which match
        *all* of the filters will be returned.

    search_metadata
        Specify a dictionary to match to the job's metadata. If any of the
        key-value pairs in this dictionary match, the job will be returned.
        Example:

        .. code-block:: bash

            salt-run jobs.list_jobs search_metadata='{"foo": "bar", "baz": "qux"}'

    search_function
        Can be passed as a string or a list. Returns jobs which match the
        specified function. Globbing is allowed. Example:

        .. code-block:: bash

            salt-run jobs.list_jobs search_function='test.*'
            salt-run jobs.list_jobs search_function='["test.*", "pkg.install"]'

        .. versionchanged:: 2015.8.8
            Multiple targets can now also be passed as a comma-separated list.
            For example:

            .. code-block:: bash

                salt-run jobs.list_jobs search_function='test.*,pkg.install'

    search_target
        Can be passed as a string or a list. Returns jobs which match the
        specified minion name. Globbing is allowed. Example:

        .. code-block:: bash

            salt-run jobs.list_jobs search_target='*.mydomain.tld'
            salt-run jobs.list_jobs search_target='["db*", "myminion"]'

        .. versionchanged:: 2015.8.8
            Multiple targets can now also be passed as a comma-separated list.
            For example:

            .. code-block:: bash

                salt-run jobs.list_jobs search_target='db*,myminion'

    start_time
        Accepts any timestamp supported by the dateutil_ Python module (if this
        module is not installed, this argument will be ignored). Returns jobs
        which started after this timestamp.

    end_time
        Accepts any timestamp supported by the dateutil_ Python module (if this
        module is not installed, this argument will be ignored). Returns jobs
        which started before this timestamp.

    .. _dateutil: https://pypi.python.org/pypi/python-dateutil

    CLI Example:

    .. code-block:: bash

        salt-run jobs.list_jobs
        salt-run jobs.list_jobs search_function='test.*' search_target='localhost' search_metadata='{"bar": "foo"}'
        salt-run jobs.list_jobs start_time='2015, Mar 16 19:00' end_time='2015, Mar 18 22:00'

    """
    returner = _get_returner(
        (__opts__["ext_job_cache"], ext_source, __opts__["master_job_cache"])
    )
    if display_progress:
        __jid_event__.fire_event(
            {"message": "Querying returner {0} for jobs.".format(returner)}, "progress"
        )
    mminion = salt.minion.MasterMinion(__opts__)

    ret = mminion.returners["{0}.get_jids".format(returner)]()

    mret = {}
    for item in ret:
        _match = True
        if search_metadata:
            _match = False
            if "Metadata" in ret[item]:
                if isinstance(search_metadata, dict):
                    for key in search_metadata:
                        if key in ret[item]["Metadata"]:
                            if ret[item]["Metadata"][key] == search_metadata[key]:
                                _match = True
                else:
                    log.info(
                        "The search_metadata parameter must be specified"
                        " as a dictionary.  Ignoring."
                    )
        if search_target and _match:
            _match = False
            if "Target" in ret[item]:
                targets = ret[item]["Target"]
                if isinstance(targets, six.string_types):
                    targets = [targets]
                for target in targets:
                    for key in salt.utils.args.split_input(search_target):
                        if fnmatch.fnmatch(target, key):
                            _match = True

        if search_function and _match:
            _match = False
            if "Function" in ret[item]:
                for key in salt.utils.args.split_input(search_function):
                    if fnmatch.fnmatch(ret[item]["Function"], key):
                        _match = True

        if start_time and _match:
            _match = False
            if DATEUTIL_SUPPORT:
                parsed_start_time = dateutil_parser.parse(start_time)
                _start_time = dateutil_parser.parse(ret[item]["StartTime"])
                if _start_time >= parsed_start_time:
                    _match = True
            else:
                log.error(
                    "'dateutil' library not available, skipping start_time "
                    "comparison."
                )

        if end_time and _match:
            _match = False
            if DATEUTIL_SUPPORT:
                parsed_end_time = dateutil_parser.parse(end_time)
                _start_time = dateutil_parser.parse(ret[item]["StartTime"])
                if _start_time <= parsed_end_time:
                    _match = True
            else:
                log.error(
                    "'dateutil' library not available, skipping end_time " "comparison."
                )

        if _match:
            mret[item] = ret[item]

    if outputter:
        return {"outputter": outputter, "data": mret}
    else:
        return mret


def list_jobs_filter(
    count, filter_find_job=True, ext_source=None, outputter=None, display_progress=False
):
    """
    List all detectable jobs and associated functions

    ext_source
        The external job cache to use. Default: `None`.

    CLI Example:

    .. code-block:: bash

        salt-run jobs.list_jobs_filter 50
        salt-run jobs.list_jobs_filter 100 filter_find_job=False

    """
    returner = _get_returner(
        (__opts__["ext_job_cache"], ext_source, __opts__["master_job_cache"])
    )
    if display_progress:
        __jid_event__.fire_event(
            {"message": "Querying returner {0} for jobs.".format(returner)}, "progress"
        )
    mminion = salt.minion.MasterMinion(__opts__)

    fun = "{0}.get_jids_filter".format(returner)
    if fun not in mminion.returners:
        raise NotImplementedError(
            "'{0}' returner function not implemented yet.".format(fun)
        )
    ret = mminion.returners[fun](count, filter_find_job)

    if outputter:
        return {"outputter": outputter, "data": ret}
    else:
        return ret


def print_job(jid, ext_source=None):
    """
    Print a specific job's detail given by its jid, including the return data.

    CLI Example:

    .. code-block:: bash

        salt-run jobs.print_job 20130916125524463507
    """
    ret = {}

    returner = _get_returner(
        (__opts__["ext_job_cache"], ext_source, __opts__["master_job_cache"])
    )
    mminion = salt.minion.MasterMinion(__opts__)

    try:
        job = mminion.returners["{0}.get_load".format(returner)](jid)
        ret[jid] = _format_jid_instance(jid, job)
    except TypeError:
        ret[jid]["Result"] = (
            "Requested returner {0} is not available. Jobs cannot be "
            "retrieved. Check master log for details.".format(returner)
        )
        return ret
    ret[jid]["Result"] = mminion.returners["{0}.get_jid".format(returner)](jid)

    fstr = "{0}.get_endtime".format(__opts__["master_job_cache"])
    if __opts__.get("job_cache_store_endtime") and fstr in mminion.returners:
        endtime = mminion.returners[fstr](jid)
        if endtime:
            ret[jid]["EndTime"] = endtime

    return ret


def exit_success(jid, ext_source=None):
    """
    Check if a job has been executed and exit successfully

    jid
        The jid to look up.
    ext_source
        The external job cache to use. Default: `None`.

    CLI Example:

    .. code-block:: bash

        salt-run jobs.exit_success 20160520145827701627
    """
    ret = dict()

    data = list_job(jid, ext_source=ext_source)

    minions = data.get("Minions", [])
    result = data.get("Result", {})

    for minion in minions:
        if minion in result and "return" in result[minion]:
            ret[minion] = True if result[minion]["return"] else False
        else:
            ret[minion] = False

    for minion in result:
        if "return" in result[minion] and result[minion]["return"]:
            ret[minion] = True
    return ret


def last_run(
    ext_source=None,
    outputter=None,
    metadata=None,
    function=None,
    target=None,
    display_progress=False,
):
    """
    .. versionadded:: 2015.8.0

    List all detectable jobs and associated functions

    CLI Example:

    .. code-block:: bash

        salt-run jobs.last_run
        salt-run jobs.last_run target=nodename
        salt-run jobs.last_run function='cmd.run'
        salt-run jobs.last_run metadata="{'foo': 'bar'}"
    """

    if metadata:
        if not isinstance(metadata, dict):
            log.info("The metadata parameter must be specified as a dictionary")
            return False

    _all_jobs = list_jobs(
        ext_source=ext_source,
        outputter=outputter,
        search_metadata=metadata,
        search_function=function,
        search_target=target,
        display_progress=display_progress,
    )
    if _all_jobs:
        last_job = sorted(_all_jobs)[-1]
        return print_job(last_job, ext_source)
    else:
        return False


def _get_returner(returner_types):
    """
    Helper to iterate over returner_types and pick the first one
    """
    for returner in returner_types:
        if returner and returner is not None:
            return returner


def _format_job_instance(job):
    """
    Helper to format a job instance
    """
    if not job:
        ret = {"Error": "Cannot contact returner or no job with this jid"}
        return ret

    ret = {
        "Function": job.get("fun", "unknown-function"),
        "Arguments": list(job.get("arg", [])),
        # unlikely but safeguard from invalid returns
        "Target": job.get("tgt", "unknown-target"),
        "Target-type": job.get("tgt_type", "list"),
        "User": job.get("user", "root"),
    }

    if "metadata" in job:
        ret["Metadata"] = job.get("metadata", {})
    else:
        if "kwargs" in job:
            if "metadata" in job["kwargs"]:
                ret["Metadata"] = job["kwargs"].get("metadata", {})

    if "Minions" in job:
        ret["Minions"] = job["Minions"]
    return ret


def _format_jid_instance(jid, job):
    """
    Helper to format jid instance
    """
    ret = _format_job_instance(job)
    ret.update({"StartTime": salt.utils.jid.jid_to_time(jid)})
    return ret


def _walk_through(job_dir, display_progress=False):
    """
    Walk through the job dir and return jobs
    """
    serial = salt.payload.Serial(__opts__)

    for top in os.listdir(job_dir):
        t_path = os.path.join(job_dir, top)

        for final in os.listdir(t_path):
            load_path = os.path.join(t_path, final, ".load.p")
            with salt.utils.files.fopen(load_path, "rb") as rfh:
                job = serial.load(rfh)

            if not os.path.isfile(load_path):
                continue

            with salt.utils.files.fopen(load_path, "rb") as rfh:
                job = serial.load(rfh)
            jid = job["jid"]
            if display_progress:
                __jid_event__.fire_event(
                    {"message": "Found JID {0}".format(jid)}, "progress"
                )
            yield jid, job, t_path, final



# -*- coding: utf-8 -*-
"""
    tests.support.generate-from-names-from-failed-test-reports
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    This script is meant as a stop-gap until we move to PyTest to provide a functionality similar to
    PyTest's --last-failed where PyTest only runs last failed tests.
"""
# pylint: disable=resource-leakage
from __future__ import absolute_import, print_function, unicode_literals

import argparse
import glob
import os
import sys

try:
    import xunitparser
except ImportError:
    sys.stderr.write(
        "Please install the xunitparser python package to run this script\n"
    )
    sys.stderr.flush()
    sys.exit(1)

REPO_ROOT = os.path.abspath(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--reports-dir",
        default=os.path.join(REPO_ROOT, "artifacts", "xml-unittests-output"),
        help="Path to the directory where the JUnit XML reports can be found",
    )
    parser.add_argument(
        "output_file",
        help="Path to the file containing the failed tests listing to be fed to --names-files",
    )
    options = parser.parse_args()
    total_xml_reports = 0
    failures = set()
    for fname in sorted(glob.glob(os.path.join(options.reports_dir, "*.xml"))):
        total_xml_reports += 1
        with open(fname) as rfh:
            test_suite, test_result = xunitparser.parse(rfh)
            if not test_result.errors and not test_result.failures:
                continue
            for test in test_suite:
                if test.bad:
                    failures.add("{classname}.{methodname}".format(**test.__dict__))

    if not total_xml_reports:
        parser.exit(status=1, message="No JUnit XML files were parsed")

    with open(options.output_file, "w") as wfh:
        wfh.write(os.linesep.join(sorted(failures)))

    parser.exit(status=0)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
    tests.unit.cloud
    ~~~~~~~~~~~~~~~~
"""
from __future__ import absolute_import, print_function, unicode_literals

import salt.cloud
from tests.support.helpers import slowTest
from tests.support.unit import TestCase


class CloudTest(TestCase):
    @slowTest
    def test_vm_config_merger(self):
        """
        Validate the vm's config is generated correctly.

        https://github.com/saltstack/salt/issues/49226
        """
        main = {
            "minion": {"master": "172.31.39.213"},
            "log_file": "var/log/salt/cloud.log",
            "pool_size": 10,
        }
        provider = {
            "private_key": "dwoz.pem",
            "grains": {"foo1": "bar", "foo2": "bang"},
            "availability_zone": "us-west-2b",
            "driver": "ec2",
            "ssh_interface": "private_ips",
            "ssh_username": "admin",
            "location": "us-west-2",
        }
        profile = {
            "profile": "default",
            "grains": {"meh2": "bar", "meh1": "foo"},
            "provider": "ec2-default:ec2",
            "ssh_username": "admin",
            "image": "ami-0a1fbca0e5b419fd1",
            "size": "t2.micro",
        }
        vm = salt.cloud.Cloud.vm_config("test_vm", main, provider, profile, {})
        self.assertEqual(
            {
                "minion": {"master": "172.31.39.213"},
                "log_file": "var/log/salt/cloud.log",
                "pool_size": 10,
                "private_key": "dwoz.pem",
                "grains": {
                    "foo1": "bar",
                    "foo2": "bang",
                    "meh2": "bar",
                    "meh1": "foo",
                },
                "availability_zone": "us-west-2b",
                "driver": "ec2",
                "ssh_interface": "private_ips",
                "ssh_username": "admin",
                "location": "us-west-2",
                "profile": "default",
                "provider": "ec2-default:ec2",
                "image": "ami-0a1fbca0e5b419fd1",
                "size": "t2.micro",
                "name": "test_vm",
            },
            vm,
        )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import

# Import Salt Libs
import salt.states.memcached as memcached
from salt.exceptions import CommandExecutionError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class MemcachedTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.memcached
    """

    def setup_loader_modules(self):
        return {memcached: {}}

    # 'managed' function tests: 1

    def test_managed(self):
        """
        Test to manage a memcached key.
        """
        name = "foo"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        mock_t = MagicMock(
            side_effect=[CommandExecutionError, "salt", True, True, True]
        )
        with patch.dict(
            memcached.__salt__, {"memcached.get": mock_t, "memcached.set": mock_t}
        ):
            self.assertDictEqual(memcached.managed(name), ret)

            comt = "Key 'foo' does not need to be updated"
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(memcached.managed(name, "salt"), ret)

            with patch.dict(memcached.__opts__, {"test": True}):
                comt = "Value of key 'foo' would be changed"
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(memcached.managed(name, "salt"), ret)

            with patch.dict(memcached.__opts__, {"test": False}):
                comt = "Successfully set key 'foo'"
                ret.update(
                    {
                        "comment": comt,
                        "result": True,
                        "changes": {"new": "salt", "old": True},
                    }
                )
                self.assertDictEqual(memcached.managed(name, "salt"), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure that a memcached key is not present.
        """
        name = "foo"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        mock_t = MagicMock(
            side_effect=[CommandExecutionError, "salt", None, True, True, True]
        )
        with patch.dict(
            memcached.__salt__, {"memcached.get": mock_t, "memcached.delete": mock_t}
        ):
            self.assertDictEqual(memcached.absent(name), ret)

            comt = "Value of key 'foo' ('salt') is not 'bar'"
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(memcached.absent(name, "bar"), ret)

            comt = "Key 'foo' does not exist"
            ret.update({"comment": comt})
            self.assertDictEqual(memcached.absent(name), ret)

            with patch.dict(memcached.__opts__, {"test": True}):
                comt = "Key 'foo' would be deleted"
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(memcached.absent(name), ret)

            with patch.dict(memcached.__opts__, {"test": False}):
                comt = "Successfully deleted key 'foo'"
                ret.update(
                    {
                        "comment": comt,
                        "result": True,
                        "changes": {"key deleted": "foo", "value": True},
                    }
                )
                self.assertDictEqual(memcached.absent(name), ret)



# -*- coding: utf-8 -*-
"""
tests for host state
"""

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt libs
import salt.utils.platform

# Import Salt Testing libs
from tests.support.case import ModuleCase

# Import 3rd-Party libs
HAS_LSB_RELEASE = True
try:
    import lsb_release
except ImportError:
    HAS_LSB_RELEASE = False


class CompileTest(ModuleCase):
    """
    Validate the state compiler
    """

    def test_multi_state(self):
        """
        Test the error with multiple states of the same type
        """
        ret = self.run_function("state.sls", mods="fuzz.multi_state")
        # Verify that the return is a list, aka, an error
        self.assertIsInstance(ret, list)

    def test_jinja_deep_error(self):
        """
        Test when we have an error in a execution module
        called by jinja
        """
        if salt.utils.platform.is_linux() and HAS_LSB_RELEASE:
            release = lsb_release.get_distro_information()
            if (
                release.get("ID") == "Debian"
                and int(release.get("RELEASE", "0")[0]) < 9
            ):
                self.skipTest("This test is flaky on Debian 8. Skipping.")

        ret = self.run_function("state.sls", ["issue-10010"])
        self.assertTrue(", in jinja_error" in ret[0].strip())
        self.assertTrue(ret[0].strip().endswith("Exception: hehehe"))



# -*- coding: utf-8 -*-
"""
    :codeauthor: Rupesh Tare <rupesht@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.monit as monit

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class MonitTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.aptpkg
    """

    def setup_loader_modules(self):
        return {monit: {}}

    def test_start(self):
        """
        Test for start
        """
        with patch.dict(monit.__salt__, {"cmd.retcode": MagicMock(return_value=False)}):
            self.assertTrue(monit.start("name"))

    def test_stop(self):
        """
        Test for Stops service via monit
        """
        with patch.dict(monit.__salt__, {"cmd.retcode": MagicMock(return_value=False)}):
            self.assertTrue(monit.stop("name"))

    def test_restart(self):
        """
        Test for Restart service via monit
        """
        with patch.dict(monit.__salt__, {"cmd.retcode": MagicMock(return_value=False)}):
            self.assertTrue(monit.restart("name"))

    def test_unmonitor(self):
        """
        Test for Unmonitor service via monit
        """
        with patch.dict(monit.__salt__, {"cmd.retcode": MagicMock(return_value=False)}):
            self.assertTrue(monit.unmonitor("name"))

    def test_monitor(self):
        """
        Test for monitor service via monit
        """
        with patch.dict(monit.__salt__, {"cmd.retcode": MagicMock(return_value=False)}):
            self.assertTrue(monit.monitor("name"))

    def test_summary(self):
        """
        Test for Display a summary from monit
        """
        mock = MagicMock(side_effect=["daemon is not running", "A\nB\nC\nD\nE"])
        with patch.dict(monit.__salt__, {"cmd.run": mock}):
            self.assertEqual(
                monit.summary(), {"monit": "daemon is not running", "result": False}
            )

            self.assertEqual(monit.summary(), {})

    def test_status(self):
        """
        Test for Display a process status from monit
        """
        with patch.dict(monit.__salt__, {"cmd.run": MagicMock(return_value="Process")}):
            self.assertEqual(monit.status("service"), "No such service")

    def test_reload(self):
        """
        Test for Reload configuration
        """
        mock = MagicMock(return_value=0)
        with patch.dict(monit.__salt__, {"cmd.retcode": mock}):
            self.assertTrue(monit.reload_())

    def test_version(self):
        """
        Test for Display version from monit -V
        """
        mock = MagicMock(return_value="This is Monit version 5.14\nA\nB")
        with patch.dict(monit.__salt__, {"cmd.run": mock}):
            self.assertEqual(monit.version(), "5.14")

    def test_id(self):
        """
        Test for Display unique id
        """
        mock = MagicMock(return_value="Monit ID: d3b1aba48527dd599db0e86f5ad97120")
        with patch.dict(monit.__salt__, {"cmd.run": mock}):
            self.assertEqual(monit.id_(), "d3b1aba48527dd599db0e86f5ad97120")

    def test_reset_id(self):
        """
        Test for Regenerate a unique id
        """
        expected = {"stdout": "Monit id d3b1aba48527dd599db0e86f5ad97120 and ..."}
        mock = MagicMock(return_value=expected)
        with patch.dict(monit.__salt__, {"cmd.run_all": mock}):
            self.assertEqual(monit.id_(reset=True), "d3b1aba48527dd599db0e86f5ad97120")

    def test_configtest(self):
        """
        Test for Check configuration syntax
        """
        excepted = {"stdout": "Control file syntax OK", "retcode": 0, "stderr": ""}
        mock = MagicMock(return_value=excepted)
        with patch.dict(monit.__salt__, {"cmd.run_all": mock}):
            self.assertTrue(monit.configtest()["result"])
            self.assertEqual(monit.configtest()["comment"], "Syntax OK")

    def test_validate(self):
        """
        Test for Check all services are monitored
        """
        mock = MagicMock(return_value=0)
        with patch.dict(monit.__salt__, {"cmd.retcode": mock}):
            self.assertTrue(monit.validate())
